from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from pymatgen.electronic_structure.core import Spin, OrbitalType
from pymatgen.io.vasp.outputs import Oszicar, Outcar, Vasprun
from pymatgen.util.plotting import pretty_plot as get_publication_quality_plot

# from pymatgen.util.plotting_utils import get_publication_quality_plot
import matplotlib, yaml, os
from pymatgen.core.periodic_table import Element
from pymatgen.io.vasp.inputs import Potcar, Incar, Kpoints
from pymatgen.electronic_structure.plotter import (
    BSPlotterProjected,
    BSPlotter,
    DosPlotter,
)
import matplotlib

matplotlib.use("Agg")
import math, glob
from pymatgen.core.structure import Structure
import numpy as np
from pymatgen.analysis.elasticity.elastic import ElasticTensor


def ip_optics(vrun="", filename=".", extra="", plot=False):
    """
    Optoelectronic properties derived from dielectric function 

    See http://www.wien2k.at/reg_user/textbooks/WIEN2k_lecture-notes_2013/optic_handout.pdf
    http://www.phys.ufl.edu/~tanner/Wooten-OpticalPropertiesOfSolids-2up.pdf

    Args:
       vrun: path to vasprun.xml
    Reurns:
          info: dictionary with optoelectronic information
          energy: energy bins generally 0 to 40 eV
          real_part: real part of dielectric function
          imag_part: imaginary part of dielectric function
          absorption: absorption coefficient in cm^-1
       
    TODO: shorten the code with multi-dimensional np array   
    """

    info = {}

    storedir = filename

    run = Vasprun(vrun, occu_tol=1e-2)
    erange = len(run.dielectric[0])
    en = []
    realx = []
    imagx = []
    absorpx = []
    refrx = []
    reflx = []
    eelsx = []
    extcx = []
    opt_conx = []
    realy = []
    imagy = []
    absorpy = []
    refry = []
    refly = []
    eelsy = []
    extcy = []
    opt_cony = []
    realz = []
    imagz = []
    absorpz = []
    refrz = []
    reflz = []
    eelsz = []
    extcz = []
    opt_conz = []
    H = 4.13566733 * (10 ** (-15))
    # c0=2.99792458
    c0 = 2.99792458 * (math.pow(10, 8))
    for i in range(0, erange - 1):
        en.append(run.dielectric[0][i])
        realx.append(run.dielectric[1][i][0])
        imagx.append(run.dielectric[2][i][0])
        ab_valx = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][0]
                        + math.sqrt(
                            (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                            + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpx.append(ab_valx)
        refr_valx = float(
            math.sqrt(
                (run.dielectric[1][i][0])
                + math.sqrt(
                    (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                    + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                )
            )
        ) / float(1.4142)
        refrx.append(refr_valx)
        eels_valx = float(run.dielectric[2][i][0]) / float(
            (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
            + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
        )
        eelsx.append(eels_valx)
        extc_valx = float(
            math.sqrt(
                -(run.dielectric[1][i][0])
                + math.sqrt(
                    (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                    + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                )
            )
        ) / float(1.4142)
        extcx.append(extc_valx)
        refl_valx = ((refr_valx - 1) * (refr_valx - 1) + extc_valx * extc_valx) / (
            (refr_valx + 1) * (refr_valx + 1) + extc_valx * extc_valx
        )
        reflx.append(refl_valx)
        opt_valx = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][0])
        )
        opt_conx.append(opt_valx)

        realy.append(run.dielectric[1][i][1])
        imagy.append(run.dielectric[2][i][1])

        ab_valy = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][1]
                        + math.sqrt(
                            (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                            + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpy.append(ab_valy)
        refr_valy = float(
            math.sqrt(
                (run.dielectric[1][i][1])
                + math.sqrt(
                    (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                    + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                )
            )
        ) / float(1.4142)
        refry.append(refr_valy)
        eels_valy = float(run.dielectric[2][i][1]) / float(
            (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
            + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
        )
        eelsy.append(eels_valy)
        extc_valy = float(
            math.sqrt(
                -(run.dielectric[1][i][1])
                + math.sqrt(
                    (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                    + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                )
            )
        ) / float(1.4142)
        extcy.append(extc_valy)
        refl_valy = ((refr_valy - 1) * (refr_valy - 1) + extc_valy * extc_valy) / (
            (refr_valy + 1) * (refr_valy + 1) + extc_valy * extc_valy
        )
        refly.append(refl_valy)
        opt_valy = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][1])
        )
        opt_cony.append(opt_valy)

        realz.append(run.dielectric[1][i][2])
        imagz.append(run.dielectric[2][i][2])

        ab_valz = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][2]
                        + math.sqrt(
                            (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                            + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpz.append(ab_valz)
        refr_valz = float(
            math.sqrt(
                (run.dielectric[1][i][2])
                + math.sqrt(
                    (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                    + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                )
            )
        ) / float(1.4142)
        refrz.append(refr_valz)
        eels_valz = float(run.dielectric[2][i][2]) / float(
            (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
            + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
        )
        eelsz.append(eels_valz)
        extc_valz = float(
            math.sqrt(
                -(run.dielectric[1][i][2])
                + math.sqrt(
                    (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                    + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                )
            )
        ) / float(1.4142)
        extcz.append(extc_valz)
        refl_valz = ((refr_valz - 1) * (refr_valz - 1) + extc_valz * extc_valz) / (
            (refr_valz + 1) * (refr_valz + 1) + extc_valz * extc_valz
        )
        reflz.append(refl_valz)
        opt_valz = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][2])
        )
        opt_conz.append(opt_valz)

    if plot == True:
        # plt.close()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, realx, linewidth=2, label=r"$\epsilon_1x$")
        plt.plot(en, realy, linewidth=2, label=r"$\epsilon_1y$")
        plt.plot(en, realz, linewidth=2, label=r"$\epsilon_1z$")
        plt.legend(prop={"size": 26})
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Dielec. function (Real part)")
        filename = str(storedir) + str("/") + str("Real") + str(extra) + str(".png")
        plt.tight_layout()
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, imagx, linewidth=2, label=r"$\epsilon_2x$")
        plt.plot(en, imagy, linewidth=2, label=r"$\epsilon_2y$")
        plt.plot(en, imagz, linewidth=2, label=r"$\epsilon_2z$")
        plt.xlabel("Energy (eV)")
        plt.xlim([0, 30])
        plt.ylabel("Dielec. function(Imag part)")
        filename = str(storedir) + str("/") + str("Imag") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, refrx, linewidth=2, label=r"$n_x$")
        plt.plot(en, refry, linewidth=2, label=r"$n_y$")
        plt.plot(en, refrz, linewidth=2, label=r"$n_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Refractive index")
        filename = str(storedir) + str("/") + str("Refr") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, extcx, linewidth=2, label=r"$k_x$")
        plt.plot(en, extcy, linewidth=2, label=r"$k_y$")
        plt.plot(en, extcz, linewidth=2, label=r"$k_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Extinction Coefficient")
        filename = str(storedir) + str("/") + str("Extc") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, absorpx, linewidth=2, label=r"$\alpha_x$")
        plt.plot(en, absorpy, linewidth=2, label=r"$\alpha_y$")
        plt.plot(en, absorpz, linewidth=2, label=r"$\alpha_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Absorption coefficient")
        filename = str(storedir) + str("/") + str("Absorp") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, eelsx, linewidth=2, label=r"$e_x$")
        plt.plot(en, eelsy, linewidth=2, label=r"$e_y$")
        plt.plot(en, eelsz, linewidth=2, label=r"$e_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Energy loss spectrum")
        filename = str(storedir) + str("/") + str("ELS") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, opt_conx, linewidth=2, label=r"$\sigma_x$")
        plt.plot(en, opt_cony, linewidth=2, label=r"$\sigma_y$")
        plt.plot(en, opt_conz, linewidth=2, label=r"$\sigma_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Optical conductivity (Real part)")
        filename = str(storedir) + str("/") + str("Opt_con") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

    info["absorption"] = [absorpx, absorpy, absorpz]
    info["energy"] = en
    info["real_part"] = [realx, realy, realz]
    info["imag_part"] = [imagx, imagy, imagz]
    info["refr_index"] = [refrx, refry, refrz]
    info["extinc"] = [extcx, extcy, extcz]
    info["reflectance"] = [reflx, refly, reflz]
    info["EELS"] = [eelsx, eelsy, eelsz]
    info["opt_conduct"] = [opt_conx, opt_cony, opt_conz]
    print(info["real_part"][0][0])
    return info


if __name__ == "__main__":
    run = path = str(
        os.path.join(
            os.path.dirname(__file__),
            "../vasp/examples/SiOptb88/MAIN-MBJ-bulk@mp_149/vasprun.xml",
        )
    )
    ep = ip_optics(vrun=run)
    print(ep)



from jarvis.sklearn.build_models import isfloat, jdata, regr_scores, plot_learning_curve
import numpy as np


def test_isfloat():
    assert isfloat(5.5) == True


def test_jdata():
    X, Y, jid = jdata(prop="form_enp")
    assert len(X) == 24759


def test_reg_score():
    x = np.array([1, 2, 3, 4])
    y = np.array([2, 4, 6, 8])
    info = regr_scores(x, y)
    assert info["mae"] == 2.5



from pymatgen.io.vasp.inputs import Incar, Poscar
from pymatgen.io.vasp.inputs import Potcar
import yaml
import os

# os.environ["VASP_PSP_DIR"] = "/home/knc6/Software/VASP-POTENTIAL"
pot_yaml = str(
    os.path.join(os.path.dirname(__file__), "..", "vasp", "Special_POTCAR.yaml")
)
with open(pot_yaml, "r") as f:
    doc = yaml.load(f)
    pots = doc["POTCAR"]
# mat = Poscar.from_file("POSCAR")
# new_symb = []
# for el in mat.site_symbols:
#    new_symb.append(pots[el])
# potcar = Potcar(symbols=new_symb, functional="PBE")
# potcar.write_file("POTCAR")



from sklearn.model_selection import train_test_split
from sklearn.preprocessing import label_binarize
from sklearn.preprocessing import StandardScaler
from sklearn.feature_selection import VarianceThreshold
from sklearn.pipeline import Pipeline
from jarvis.sklearn.build_models import jdata
import lightgbm as lgb


def regression(X=[], Y=[]):
    lgbm = lgb.LGBMRegressor(
        n_estimators=100,
        learning_rate=0.1,
        max_depth=5,
        num_leaves=100,
        objective="regression",
        n_jobs=-1,
        verbose=-1,
    )

    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        X, Y, jid, random_state=1, test_size=0.1
    )
    pipe = Pipeline(
        [
            ("stdscal", StandardScaler()),
            ("vart", VarianceThreshold(1e-4)),
            ("est", lgbm),
        ]
    )
    if preprocess == True:
        model = pipe
    else:
        model = lgbm
    model.fit(X_train, y_train)
    pred = model.predict(X_test)
    reg_sc = regr_scores(y_test, pred)
    if plot == True:
        plt.plot(reg_sc["pred"], reg_sc["test"], ".", label=str(type(i).__name__)[0:4])
    print(type(i).__name__, round(reg_sc["mae"], 3), round(reg_sc["rmse"], 3))
    if plot == True:
        plt.legend()
        plt.xlabel("DFT")
        plt.ylabel("ML")


if __name__ == "__main__":
    X, Y, jid = jdata(prop="exfoliation_en")
    regression(X=X, Y=Y)



from monty.json import MontyEncoder, MontyDecoder
from monty.serialization import loadfn, dumpfn
import os, requests

def get_ff_eneleast():
    jff1 = str(os.path.join(os.path.dirname(__file__), "jff1.json"))
    if not os.path.isfile(jff1):
        r = requests.get("https://ndownloader.figshare.com/files/10307139")
        f = open(jff1, "wb")
        f.write(r.content)
        f.close()
    data_ff1 = loadfn(jff1, cls=MontyDecoder)
    return data_ff1

def get_3d_dataset():
    j3d = str(os.path.join(os.path.dirname(__file__), "j3d.json"))
    if not os.path.isfile(j3d):
        r = requests.get("https://ndownloader.figshare.com/files/15475826")
        f = open(j3d, "wb")
        f.write(r.content)
        f.close()
    data_3d = loadfn(j3d, cls=MontyDecoder)
    return data_3d


def get_2d_dataset():
    j2d = str(os.path.join(os.path.dirname(__file__), "j2d.json"))
    if not os.path.isfile(j2d):
        r = requests.get("https://ndownloader.figshare.com/files/15475985")
        f = open(j2d, "wb")
        f.write(r.content)
    data_2d = loadfn(j2d, cls=MontyDecoder)
    return data_2d


def get_ml_dataset():
    jml = str(os.path.join(os.path.dirname(__file__), "jml.json"))
    if not os.path.isfile(jml):
        r = requests.get("https://ndownloader.figshare.com/files/12533579")
        f = open(jml, "wb")
        f.write(r.content)
    data_ml = loadfn(jml, cls=MontyDecoder)
    return data_ml


if __name__ == "__main__":

    data_2d = get_2d_dataset()
    print(len(data_2d))
    data_3d = get_3d_dataset()
    print(len(data_3d))
    data_ml = get_ml_dataset()
    print(len(data_ml))
    data_ff = get_ff_eneleast()
    print (len(data_ff))



import unittest
import os
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Poscar
from jarvis.sklearn.get_desc import get_comp_descp


def test_desc():
    fname = os.path.join(
        os.path.dirname(__file__), "..", "sklearn", "examples", "POSCAR"
    )
    poscar = Structure.from_file(fname)
    desc = get_comp_descp(poscar)
    assert len(desc) == 1557




from __future__ import unicode_literals, print_function

"""
Helper function for running LAMMPS
Used for defects, surface and phonon calculations
"""

from monty.json import MontyEncoder
from numpy import matrix
import shutil, time
from ase import *
import numpy as np

try:
    from phonopy import Phonopy
    from phonopy.file_IO import parse_FORCE_CONSTANTS, write_FORCE_CONSTANTS
    from phonopy.structure.atoms import Atoms as PhonopyAtoms
    from phonopy.structure.atoms import Atoms as PhonopyAtoms
except:
    pass
import glob, fileinput
from pymatgen.core.structure import Structure
from pymatgen.io.vasp.inputs import Incar, Poscar
from pymatgen.core.surface import (
    Slab,
    SlabGenerator,
    generate_all_slabs,
    get_symmetrically_distinct_miller_indices,
)
from ase.lattice.surface import surface
from pymatgen.ext.matproj import MPRester
import operator
from pymatgen.core.lattice import Lattice
from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from jarvis.point_defect.vacancy import vac_antisite_def_struct_gen, chempot_struct
from jarvis.plane_defect.surface import pmg_surfer, surfer
import numpy as np, time, json
import sys, os, subprocess, socket
from pymatgen.io.ase import AseAtomsAdaptor
from jarvis.tools.zipit import ZipDir
from jarvis.phonopy.phonon import get_phonopy_atoms
import fortranformat as fform
from pymatgen.core.structure import Structure
from ase.calculators.lammpsrun import LAMMPS, prism
try:
    input_nobox = os.environ["input_nobox"]
    input_box = os.environ["input_box"]
    lammps_exec = os.environ["lammps_exec"]
except:
    pass


def write_lammps_data(structure=None, file="", write_tmp_file=True):
    """
        write lammps structure data
        from ase with custom modifications

        Args:
            structure: Structure object
            file:  intended file to write in
        """
    structure.sort()
    if write_tmp_file == True:
        structure.to(fmt="poscar", filename="new_pymatgen_slab.vasp")
    atoms = AseAtomsAdaptor().get_atoms(structure)

    f = open(file, "w")
    f.write("datafile (written by JARVIS-FF) \n\n")
    symbols = atoms.get_chemical_symbols()
    import ase.io.vasp

    # ase.io.vasp.write_vasp("POSCAR.1x1x1",atoms, label='444supercell',direct=True,sort=False)
    n_atoms = len(symbols)
    f.write("%d \t atoms \n" % n_atoms)
    species = [tos for tos in Poscar(structure).site_symbols]
    # species = [tos.symbol
    #               for tos in structure.types_of_specie]
    n_atom_types = len(species)
    print("species", species)
    f.write("%d  atom types\n" % n_atom_types)
    p = prism(atoms.get_cell())
    xhi, yhi, zhi, xy, xz, yz = p.get_lammps_prism_str()
    f.write("0.0 %s  xlo xhi\n" % xhi)
    f.write("0.0 %s  ylo yhi\n" % yhi)
    f.write("0.0 %s  zlo zhi\n" % zhi)
    f.write("%s %s %s  xy xz yz\n" % (xy, xz, yz))
    f.write("\n\n")
    f.write("Atoms \n\n")
    for i, r in enumerate(map(p.pos_to_lammps_str, atoms.get_positions())):
        c = 0.0
        s = species.index(symbols[i]) + 1

        f.write("%6d %3d %6f %s %s %s\n" % ((i + 1, s, c) + tuple(r)))
    f.close()


def write_lammps_in(
    structure=None,
    lammps_in="init.mod",
    lammps_in1="potential.mod",
    lammps_in2="in.main",
    lammps_trj=None,
    lammps_data=None,
    parameters={},
):
    """
        write lammps input file
        from ase with custom modifications
        LAMMPS input is devided into three parts

        Args:
            structure: Structure object
            lammps_in: generally"init.mod", with unit and conversion factor information
            lammps_in1: generally "potential.mod", with force-field/potential style and element tyoe information
            lammps_in2: generally "in.elastic", a generic main input file to be fed in LAMMPS usin lmp_*<...,parameters['exec']
            parameters: input parameters
            
        """
    structure.sort()
    f = open(lammps_in, "w")
    f1 = open(lammps_in1, "w")  # potential.mod
    f2 = open(lammps_in2, "w")
    f.write(
        ('variable dump_file string "%s"\n' % lammps_trj)
        + ("variable up  equal 1.0e-6\n")
        + ("variable cfac  equal 1.0e-4\n")
        + ("variable cunits  string GPa\n")
        + ("variable etol  equal 0\n")
        + ("variable ftol  equal 1.0e-10\n")
        + ("variable maxiter equal 1000\n")
        + ("variable maxeval equal 10000\n")
        + ("variable dmax equal 1.0e-2\n")
        + ('variable data_file string "%s"\n' % "data")
    )
    atoms = AseAtomsAdaptor().get_atoms(structure)
    pbc = atoms.get_pbc()
    if "control_file" in parameters:
        f2.write("include %s \n" % parameters["control_file"])
    if "units" in parameters:
        f.write("units %s \n" % parameters["units"])
    else:
        f.write("units metal \n")
    if "atom_style" in parameters:
        f.write("atom_style %s \n" % parameters["atom_style"])
    else:
        f.write("atom_style atomic \n")
    if "boundary" in parameters:
        f.write("boundary %s \n" % parameters["boundary"])
    else:
        f.write("boundary %c %c %c \n" % tuple("sp"[x] for x in pbc))
    f.write("atom_modify sort 0 0.0 \n")
    for key in ("neighbor", "newton"):
        if key in parameters:
            f.write("%s %s \n" % (key, parameters[key]))
    f.write("\n")
    # If no_data_file,
    # write the simulation box and the atoms
    species = [tos for tos in Poscar(structure).site_symbols]
    # species = [tos.symbol for tos in structure.types_of_specie]
    n_atom_types = len(species)
    species_i = dict([(s, i + 1) for i, s in enumerate(species)])
    f.write("read_data %s\n" % "data")
    # interaction
    f.write("\n### interactions \n")
    if "lib" in parameters:
        lib = parameters["lib"]
        f1.write("%s \n" % lib)
    if ("pair_style" in parameters) and ("pair_coeff" in parameters):
        pair_style = parameters["pair_style"]
        f1.write("pair_style %s \n" % pair_style)
        symbols = atoms.get_chemical_symbols()
        # species = [tos.symbol.replace("Mo","M") for tos in structure.types_of_specie] #For REBO Mo-S
        # species = [tos.symbol for tos in structure.types_of_specie]
        print("site symbolss", Poscar(structure).site_symbols)
        species = [tos for tos in Poscar(structure).site_symbols]
        if parameters["pair_style"] == "rebomos":

            species = [tos.replace("Mo", "M") for tos in Poscar(structure).site_symbols]
        tag = ""
        for i in species:
            tag = tag + " " + i
        pair_coef = "* * " + str(parameters["pair_coeff"]) + " " + tag
        f1.write("pair_coeff %s \n" % pair_coef)

        masses = []
        for m in atoms.get_masses():
            if m not in masses:
                masses.append(m)
        count = 0
        for i in masses:
            count = count + 1
            f.write("mass" + " " + str(count) + " " + str(i) + "\n")
    else:
        # default interaction
        f.write("pair_style lj/cut 2.5 \n" + "pair_coeff * * 1 1 \n" + "mass * 1.0 \n")
    f1.write("neighbor 1.0 nsq\n")
    f1.write("neigh_modify once no every 1 delay 0 check yes\n")
    if "min" not in parameters:
        f1.write("min_style  cg\n")
        f1.write("min_modify           dmax ${dmax} line quadratic\n")
    f1.write("thermo          1\n")
    f1.write(
        "thermo_style custom step temp press cpu pxx pyy pzz pxy pxz pyz ke pe etotal vol lx ly lz atoms\n"
    )
    # f1.write('thermo_style custom step temp pe press pxx pyy pzz pxy pxz pyz lx ly lz vol\n' )
    f1.write("thermo_modify norm no\n")
    #   if 'thermo_style' in parameters:
    #       f.write('thermo_style %s\n' % parameters['thermo_style'])
    #   else:
    #       f.write(('thermo_style custom %s\n') %
    #               (' '.join(self._custom_thermo_args)))
    #   if 'thermo_modify' in parameters:
    #       f.write('thermo_modify %s\n' % parameters['thermo_modify'])
    #   else:
    #       f.write('thermo_modify flush yes\n')
    #   if 'thermo' in parameters:
    #       f.write('thermo %s\n' % parameters['thermo'])
    #   else:
    #       f.write('thermo 1\n')
    #   if 'minimize' in parameters:
    #       f.write('minimize %s\n' % parameters['minimize'])
    #   if 'run' in parameters:
    #       f.write('run %s\n' % parameters['run'])
    #   if not (('minimize' in parameters) or ('run' in parameters)):
    #       f.write('run 0\n')
    #   if 'dump' in parameters:
    #       f.write('dump %s\n' % parameters['dump'])
    #   else:
    #       f.write('dump dump_all all custom 1 %s id type x y z vx vy vz fx fy fz\n' % lammps_trj)
    #   f.write('print __end_of_ase_invoked_calculation__\n')
    #   f.write('log /dev/stdout\n')
    if "fix" in parameters:
        if parameters["fix"]:
            for i in parameters["fix"]:
                f1.write("fix %s\n" % i)
    f.close()
    f1.close()
    f2.close()


def analyz_loge(log="log.lammps"):
    import sys

    """ 
    Analyzes log.lammps file,
    Please note, the output format heavily depends on the input file
    A generic inpu is taken here

    Args:
        log: path to log.lammps file
    Returns:
          en: energy/atom
          press: pressure
          toten: total energy
          cij: elastic constants
    """
    en = 0
    press = 0
    c11 = 0
    c22 = 0
    c33 = 0
    c44 = 0
    c55 = 0
    c66 = 0
    c12 = 0
    c13 = 0
    c23 = 0
    c14 = 0
    c15 = 0
    c16 = 0
    c14 = 0
    c24 = 0
    c25 = 0
    c26 = 0
    c34 = 0
    c35 = 0
    c36 = 0
    c45 = 0
    c46 = 0
    c56 = 0
    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if "Loop time of" in line:
                toten = float(lines[i - 1].split()[12])
                press = float(lines[i - 1].split()[2])
                press = float(press) * 0.0001
                en = float(lines[i - 1].split()[12]) / float(lines[i - 1].split()[17])
                break
        logfile.close()
    except:
        pass
    try:
        logfile = open(log, "r")
        lines = logfile.read().splitlines()
        for i, line in enumerate(lines):
            if 'print "Elastic Constant C11all = ${C11all} ${cunits}"' in line:
                c11 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C22all = ${C22all} ${cunits}"' in line:
                c22 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C33all = ${C33all} ${cunits}"' in line:
                c33 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C12all = ${C12all} ${cunits}"' in line:
                c12 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C13all = ${C13all} ${cunits}"' in line:
                c13 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C23all = ${C23all} ${cunits}"' in line:
                c23 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C44all = ${C44all} ${cunits}"' in line:
                c44 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C55all = ${C55all} ${cunits}"' in line:
                c55 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C66all = ${C66all} ${cunits}"' in line:
                c66 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C14all = ${C14all} ${cunits}"' in line:
                c14 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C16all = ${C16all} ${cunits}"' in line:
                c16 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C24all = ${C24all} ${cunits}"' in line:
                c24 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C25all = ${C25all} ${cunits}"' in line:
                c25 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C26all = ${C26all} ${cunits}"' in line:
                c26 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C34all = ${C34all} ${cunits}"' in line:
                c34 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C35all = ${C35all} ${cunits}"' in line:
                c35 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C36all = ${C36all} ${cunits}"' in line:
                c36 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C45all = ${C45all} ${cunits}"' in line:
                c45 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C46all = ${C46all} ${cunits}"' in line:
                c46 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
            if 'print "Elastic Constant C56all = ${C56all} ${cunits}"' in line:
                c56 = ((str((lines[i + 1])).split("=")[1]).split("GPa"))[0]
        logfile.close()
    except:
        pass
    return (
        round(en, 2),
        round(press, 2),
        float(toten),
        round(float(c11), 1),
        round(float(c22), 1),
        round(float(c33), 1),
        round(float(c12), 1),
        round(float(c13), 1),
        round(float(c23), 1),
        round(float(c44), 1),
        round(float(c55), 1),
        round(float(c66), 1),
        round(float(c14), 1),
        round(float(c16), 1),
        round(float(c24), 1),
        round(float(c25), 1),
        round(float(c26), 1),
        round(float(c34), 1),
        round(float(c35), 1),
        round(float(c36), 1),
        round(float(c45), 1),
        round(float(c46), 1),
        round(float(c56), 1),
    )


def read_data(data=None, ff=None):
    """
    Read LAMMPS data file

    Args:
        data: data file path
        ff: potential.mod/potential information file path
    Returns:
          struct: Structure object
    """
    pot_file = open(ff, "r")
    lines = pot_file.read().splitlines()
    symb = []
    count = 0
    from pymatgen.core.periodic_table import Element

    for i, line in enumerate(lines):
        if "pair_coeff" in line.split():
            sp = line.split()
            print("spsplit", sp, os.getcwd())
            for el in sp:
                try:
                    if Element(el):
                        # if el=='M':
                        #    el='Mo'
                        # count=count+1
                        # if count >4:
                        symb.append(el)
                except:
                    pass
    print("symb=", symb)

    f = open(data, "r")
    lines = f.read().splitlines()
    for i, line in enumerate(lines):
        if "atoms" in line.split():
            natoms = int(line.split()[0])
        if "types" in line.split():
            print(line)
            ntypes = int(line.split()[0])
        if "xlo" in line.split():
            xlo = float(line.split()[0])
            xhi = float(line.split()[1])
        if "ylo" in line.split():
            ylo = float(line.split()[0])
            yhi = float(line.split()[1])
        if "zlo" in line.split():
            zlo = float(line.split()[0])
            zhi = float(line.split()[1])
        if "xy" in line.split():
            xy = float(line.split()[0])
            xz = float(line.split()[1])
            yz = float(line.split()[2])
    if len(symb) != ntypes:
        print("Something wrong in atom type assignment", len(symb), ntypes)
        sys.exit()
    lat = Lattice([[xhi - xlo, 0.0, 0.0], [xy, yhi - ylo, 0.0], [xz, yz, zhi - zlo]])
    typ = np.empty((natoms), dtype="S20")
    x = np.zeros((natoms))
    y = np.zeros((natoms))
    z = np.zeros((natoms))
    q = np.zeros((natoms))
    coords = list()
    for i, line in enumerate(lines):
        if "Atoms" in line.split():
            for j in range(0, natoms):
                # print int(((lines[j+2]).split()[1]))-1
                typ[j] = symb[int(((lines[i + j + 2]).split()[1])) - 1]
                q[j] = (lines[i + j + 2]).split()[2]
                x[j] = (lines[i + j + 2]).split()[3]
                y[j] = (lines[i + j + 2]).split()[4]
                z[j] = (lines[i + j + 2]).split()[5]
                coords.append([x[j], y[j], z[j]])
    f.close()
    # print ("info",(typ),'coo',(coords),'latt',lat)
    pot_file.close()
    typ_sp = [str(i, "utf-8") for i in typ]
    struct = Structure(lat, typ_sp, coords, coords_are_cartesian=True)
    # print struct
    # finder = SpacegroupAnalyzer(struct)
    # num=finder.get_spacegroup_symbol()
    # print(num)
    return struct


def smart_vac(strt=None, parameters=None):
    """
    Function to get all vacancy formation energies

    Args:
        strt: Structure object
        parameters: parameters with LAMMPS inputs
    """
    parameters["control_file"] = input_nobox  #'/users/knc6/inelast_nobox.mod'
    vac_arr = []
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()

    cellmax = 2  # int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    ase_atoms = ase_atoms * (cellmax, cellmax, cellmax)
    # if len(ase_atoms) >200:
    #   cellmax=1
    # else:
    #   cellmax=2
    # cellmax=int(mat_cvn.composition.num_atoms)+int(mat_cvn.ntypesp)#5
    print("type of trt is= celmmax", type(strt), cellmax)
    try:
        print("int(strt.composition.num_atoms)", int(strt.composition.num_atoms))
        print(int(strt.ntypesp))
    except:
        pass
    # cellmax=int(strt.composition.num_atoms)+int(strt.ntypesp)
    vac_done = 0
    tol = 0.01  # change 0.1
    try:
        vac = vac_antisite_def_struct_gen(cellmax=cellmax, struct=strt)
    except:
        print("Failed for v_1", os.getcwd())
        pass
    def_list = [100000 for y in range(len(vac) - 1)]
    while vac_done != 1:
        try:
            vac = vac_antisite_def_struct_gen(cellmax=cellmax, struct=strt)
        except:
            print("Failed for v_2", os.getcwd())
            pass
        if vac not in vac_arr:
            try:
                vac_arr.append(vac)
                print(
                    "in smart_vac(strt=None), cellmax,vac,vac_done=",
                    cellmax,
                    vac,
                    vac_done,
                )
                def_list2, header_list = def_energy(vac=vac, parameters=parameters)
                diff = matrix(def_list) - matrix(def_list2)
                diff_arr = np.array(diff).flatten()
                print("in smart_vac(strt=None diff_arr=", diff_arr)
                print("sleepig for 5")
                # time.sleep(5)
            except:
                print("Failed for v_3", os.getcwd())
                pass
            if any(diff_arr) > tol:
                # for el in diff_arr:
                #     if abs(el)>tol :
                # print ("in smart_vac(strt=None abs_el=",abs(el))
                vac_done = 0
                cellmax = cellmax + 1
                ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
                ase_atoms = ase_atoms * (cellmax, cellmax, cellmax)
                if len(ase_atoms) > 50:
                    vac_done = 1
                    break
                def_list = def_list2
            else:
                vac_done = 1
    #        cellmax=cellmax+1
    return def_list, header_list


def smart_surf(strt=None, parameters=None, layers=3, tol=0.5):
    """
    Function to get all surface energies

    Args:
        strt: Structure object
        parameters: parameters with LAMMPS inputs
        layers: starting number of layers
        tol: surface energy tolerance for convergence
    Returns:
          surf_list: list of surface energies
          surf_header_list: list of surface names

    """
    parameters["control_file"] = input_nobox  #'/users/knc6/inelast_nobox.mod'
    sg_mat = SpacegroupAnalyzer(strt)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    mat_cvn.sort()
    layers = 3
    indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
    ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
    for i in indices:
        ase_slab = surface(ase_atoms, i, layers)
        ase_slab.center(vacuum=15, axis=2)
        if len(ase_slab) < 50:
            layers = 3
    surf_arr = []
    surf_done = True
    try:
        surf = surfer(mat=strt, layers=layers)
        surf_list = [100000 for y in range(len(surf) - 1)]
        print("in smart_surf :surf,surf_list=", surf, surf_list)
    except:
        print("Failed at s1", os.getcwd())
        pass
    while surf_done:
        layers = layers + 1
        indices = get_symmetrically_distinct_miller_indices(mat_cvn, 1)
        ase_atoms = AseAtomsAdaptor().get_atoms(mat_cvn)
        for i in indices:
            ase_slab = surface(ase_atoms, i, layers)
            ase_slab.center(vacuum=15, axis=2)
            # if len(ase_slab) > 100:
            #   surf_done=True
            # if (ase_slab.get_cell()[2][2]) > 40:
            #   surf_done=True

        try:
            surf = surfer(mat=strt, layers=layers)
        except:

            print("Failed at s2", os.getcwd())
            pass
        if surf not in surf_arr:
            surf_arr.append(surf)
            try:
                surf_list2, surf_header_list = surf_energy(
                    surf=surf, parameters=parameters
                )
                print("in smart_surf :surf2,surf_list2=", surf_list2, surf_header_list)
                diff = matrix(surf_list) - matrix(surf_list2)
                print(
                    "in smart_surf :surf3,surf_list3=",
                    matrix(surf_list),
                    matrix(surf_list2),
                )
                diff_arr = np.array(diff).flatten()
            except:
                print("Failed for s_3", os.getcwd())
                pass
            if len(ase_slab) > 50:
                surf_done = True
                break
                # print ("layersssssssssssssssssssssssss",layers,surf_done)
                break
            if any(diff_arr) > tol:
                # for el in diff_arr:
                #    if abs(el)>tol :
                #        print ("in smart_surf :abs el=",abs(el))
                surf_done = True
                surf_list = surf_list2
            else:
                surf_done = False
    return surf_list, surf_header_list


def surf_energy(surf=[], parameters={}):
    """
    Function to get specific surface energies

    Args:
        surf: list of syrfaces
        parameters: LAMMPS input parameters
    Returns:
          surf_list: list of surface energies
          surf_header_list: list of surface names
    """
    # def surf_energy(surf=[],parameters = {'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    surf_list = []
    surf_header_list = []
    fi = str("SURF.INFO")
    f = open(fi, "w")
    for s in surf:
        enp, strt, forces = run_job(mat=s, parameters=parameters)
        # strt=Structure.from_file(contc)
        m = strt.lattice.matrix
        if s.comment.split("@")[0] == "sbulk":

            gs_energy = enp
            print("in surf_energy gs_energy for", s.comment, gs_energy)
        else:
            surf_en = (
                16
                * (
                    float(enp) * (strt.composition.num_atoms)
                    - float(strt.composition.num_atoms) * float(gs_energy)
                )
                / (2 * np.linalg.norm(np.cross(m[0], m[1])))
            )
            print("in surf_energy surf_en for", s.comment, surf_en)
            surf_list.append(surf_en)
            print(s.comment, "=", surf_en)
            line = str(s.comment) + str("=") + str(surf_en) + "\n"
            f.write(line)
    f.close()
    return surf_list, surf_header_list


def get_chem_pot(s1=None, s2=None, parameters={}):
    """
    Get chemical potential given perfect and defect structures

    Args:
        s1: perfect Structure object
        s2: defect Structure object
        parameters: LAMMPS input parameter
    Returns:
           enp: energy per atom
    """
    # def get_chem_pot(s1=None,s2=None,parameters= {'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    s1.sort()
    s2.sort()
    s3 = (set(s2)).symmetric_difference(set(s1))  # list(set(s1)-set(s2))
    # s3=[]
    # for el1 in s1:
    #     for el2 in s2:
    #         if el1 !=el2 and el1 not in s3:
    #            s3.append(el1)
    #         if el1 !=el2 and el2 not in s3:
    #            s3.append(el2)
    #
    # from pymatgen.analysis.structure_matcher import StructureMatcher
    # print "Mather",StructureMatcher().fit(s1, s2)
    print("s3   is   ", type(s1), type(s2), s3)
    uniq = []
    for q in s3:
        el = q._species.elements
        for j in el:
            print("j is ", j)
            if j not in uniq:
                uniq.append(j)
                a, b = chempot_struct(j.symbol)
                # a, b = get_struct_from_jv(j)
                p = Poscar(b)
                p.comment = str(a)
                enp, strt, forces = run_job(mat=p, parameters=parameters)
    if len(uniq) > 1:
        print("uniq problem", uniq)
    return enp


def calc_forces(mat=None, parameters={}):
    """
    Calculate forces on atoms
   
    """
    enp, strt, forces = run_job(mat=mat, parameters=parameters)
    return forces


def do_phonons(strt=None, parameters=None, c_size=15):
    """
    Setting up phonopy job using LAMMPS

    Args:
        strt: Structure object
        parameters: LAMMPS input file parameters
        c_size: cell-size 
    
    """
    # spg_strt = SpacegroupAnalyzer(strt).get_conventional_standard_structure()
    p = get_phonopy_atoms(mat=strt)
    bulk = p

    dim1 = int((float(c_size) / float(max(abs(strt.lattice.matrix[0]))))) + 1
    dim2 = int(float(c_size) / float(max(abs(strt.lattice.matrix[1])))) + 1
    dim3 = int(float(c_size) / float(max(abs(strt.lattice.matrix[2])))) + 1
    Poscar(strt).write_file("POSCAR")
    tmp = strt.copy()
    tmp.make_supercell([dim1, dim2, dim3])
    Poscar(tmp).write_file("POSCAR-Super.vasp")

    # phonon = Phonopy(get_phonopy_atoms(tmp), [[dim1, 0, 0], [0, dim2, 0], [0, 0, dim3]])  # ,
    phonon = Phonopy(bulk, [[dim1, 0, 0], [0, dim2, 0], [0, 0, dim3]])  # ,
    print("[Phonopy] Atomic displacements1:", bulk)
    print("[Phonopy] Atomic displacements2:", phonon, dim1, dim2, dim3)
    phonon.generate_displacements(distance=0.03)
    disps = phonon.get_displacements()
    print("[Phonopy] Atomic displacements3:", disps)
    for d in disps:
        print("[Phonopy]", d[0], d[1:])
    supercells = phonon.get_supercells_with_displacements()

    # Force calculations by calculator
    set_of_forces = []
    disp = 0
    for scell in supercells:
        cell = Atoms(
            symbols=scell.get_chemical_symbols(),
            scaled_positions=scell.get_scaled_positions(),
            cell=scell.get_cell(),
            pbc=True,
        )
        disp = disp + 1

    mat = Poscar(AseAtomsAdaptor().get_structure(cell))
    mat.comment = str("disp-") + str(disp)
    parameters["min"] = "skip"
    parameters["control_file"] = "/users/knc6/in.phonon"
    # a,b,forces=run_job(mat=mat,parameters={'min':'skip','pair_coeff': '/data/knc6/JARVIS-FF-NEW/ALLOY4/Mishin-Ni-Al-2009.eam.alloy', 'control_file': '/users/knc6/in.phonon', 'pair_style': 'eam/alloy', 'atom_style': 'charge'})
    a, b, forces = run_job(mat=mat, parameters=parameters)
    print("forces=", forces)
    drift_force = forces.sum(axis=0)
    print("drift forces=", drift_force)
    print("[Phonopy] Drift force:", "%11.5f" * 3 % tuple(drift_force))
    # Simple translational invariance
    for force in forces:
        force -= drift_force / forces.shape[0]
    set_of_forces.append(forces)
    phonon.produce_force_constants(forces=set_of_forces)

    write_FORCE_CONSTANTS(phonon.get_force_constants(), filename="FORCE_CONSTANTS")
    print()
    print("[Phonopy] Phonon frequencies at Gamma:")


# for i, freq in enumerate(phonon.get_frequencies((0, 0, 0))):
#    print ("[Phonopy] %3d: %10.5f THz" %  (i + 1, freq) # THz)


def def_energy(vac=[], parameters={}):
    """
    Get specific defect formation energy

    Args:
         vac: vacancy structures
         parameters: LAMMPS input parameters
    Returns:
           def_list: defect energy list
           header_list: defect names
    """
    # def def_energy(vac=[],parameters={'pair_style':'eam/alloy','pair_coeff':'/scratch/lfs/kamal/POSMAT/Automatic2/Al03.eam.alloy','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
    def_list = []
    header_list = []
    fi = str("DEF.INFO")
    f = open(fi, "w")
    for v in vac:
        enp, strt, forces = run_job(mat=v, parameters=parameters)
        # enp,contc=run_job(mat=v,incar=incar,kpoints=kpoints,jobname=str(v.comment))
        # strt=Structure.from_file(contc)
        comm = str(v.comment)
        print("running def_energy for =", comm)
        header_list.append(comm)
        if comm.split("@")[0] == "bulk":
            gs_energy = float(enp) * float(strt.composition.num_atoms)
            gs_str = strt
            print("in def_energy gs_energy for", comm, gs_energy)
        else:
            print("strt", strt)
            print("gs_str", gs_str)
            chem_pot = get_chem_pot(gs_str, strt, parameters=parameters)
            if comm.startswith("intl"):
                chem_pot = 0.0 - float(chem_pot)
            def_en = (
                (float(enp) * float(strt.composition.num_atoms))
                - float(gs_energy)
                + chem_pot
            )
            print("in def_energy def_en for", comm, def_en, "chem_pot", chem_pot)
            def_list.append(def_en)
            print(v.comment, "=", def_en)
            line = str(v.comment) + str("=") + str(def_en) + "\n"
            f.write(line)
    f.close()
    return def_list, header_list


# def main(p=None, parameters={'pair_style':'rebomos','pair_coeff':'/scratch/lfs/kamal/JARVIS/All2/MoS2/MoS.REBO.set5b','atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'}):
def main(p=None, parameters={}, c_size=10):
    """
    Master function to run LAMMPS job

    Args:
        p: Poscar object
        parameters: LAMMPS input parameters
        c_size:cell size
    """
    # p=Poscar.from_file("POSCAR")
    sg_mat = SpacegroupAnalyzer(p.structure)
    mat_cvn = sg_mat.get_conventional_standard_structure()
    dim1 = int((float(c_size) / float(max(abs(mat_cvn.lattice.matrix[0]))))) + 1
    dim2 = int(float(c_size) / float(max(abs(mat_cvn.lattice.matrix[1])))) + 1
    dim3 = int(float(c_size) / float(max(abs(mat_cvn.lattice.matrix[2])))) + 1
    cellmax = max(dim1, dim2, dim3)
    # print "dim1 dim2 dim3",dim1,dim2,dim3
    # mat_cvn.make_supercell([dim1,dim2,dim3])
    mat_pos = Poscar(mat_cvn)
    mat_pos.comment = str(p.comment)
    print(mat_pos)
    try:
        toten, final_str, forces = run_job(mat=mat_pos, parameters=parameters)
    except:
        pass
    print("p.comment issssss", p.comment)
    vac = vac_antisite_def_struct_gen(c_size=c_size, struct=final_str)
    def_list, header_list = def_energy(vac=vac, parameters=parameters)
    print(def_list, header_list)
    try:
        print("p.comment issssss", p.comment)
        vac = vac_antisite_def_struct_gen(c_size=c_size, struct=final_str)
        def_list, header_list = def_energy(vac=vac, parameters=parameters)
        print(def_list, header_list)
    except:
        pass
    try:
        surf = pmg_surfer(mat=final_str, min_slab_size=c_size, vacuum=35, max_index=3)
        surf_list, surf_header_list = surf_energy(surf=surf, parameters=parameters)
        print(surf_list, surf_header_list)
    except:
        pass
    cwd = str(os.getcwd())
    if not os.path.exists("Phonon"):
        os.mkdir("Phonon")
    os.chdir("Phonon")
    do_phonons(strt=final_str, parameters=parameters)
    os.chdir(cwd)
    try:
        cwd = str(os.getcwd())
        if not os.path.exists("Phonon"):
            os.mkdir("Phonon")
        os.chdir("Phonon")
        do_phonons(strt=final_str, parameters=parameters)
        os.chdir(cwd)
    except:
        pass
    sub_files = []
    calc = 0
    for a in glob.glob("*.json"):
        fold = a.split(".json")[0]
        cwd = os.getcwd()
        target_file = str(cwd) + str("/") + str(fold)
        dest_file = str(cwd) + str("/") + str(fold) + str(".zip")
        sub_files.append(dest_file)
        ZipDir(
            target_file,
            dest_file,
            contents=[
                "init.mod",
                "potential.mod",
                "in.elastic",
                "data",
                "log.lammps",
                "restart.equil",
                "data0",
            ],
        )
    calc = calc + 1

    target_file = str(cwd)
    dest_file = str(cwd) + str("/") + str("Calc-") + str(calc) + str(".zip")
    ZipDir(target_file, dest_file, contents=sub_files)
    for a in glob.glob("*.json"):
        fold = a.split(".json")[0]
        cwd = os.getcwd()
        dest_file = str(cwd) + str("/") + str(fold) + str(".zip")
        line = str("rm ") + str(dest_file)
        os.system(line)


def main_func(mat=None, parameters={}):
    """
    Call master job function either using mpid or Poscar object
    """
    main(p=mat, parameters=parameters)


def run_job(mat=None, parameters={}, jobname=""):
    """ 
    Generic  function for running LAMMPS job

    Args:
        mat: Poscar object
        parameters: parameters with LAMMPS input information
        jobname: a user-defined jobname
    Returns:
          en: final enery
          final_str: final structure
          forces: forces on final structure
    """
    # def run_job(mat=None,parameters = {'exec':'/cluster/bin/lmp_ctcms-14439-knc6-2','pair_style':'comb3 polar_on','pair_coeff':None,'atom_style': 'charge' ,'control_file':'/home/kamal/inelast.mod'},jobname=''):
    jobname = str(mat.comment)
    # if jobname.startswith('bulk') or jobname.startswith('sbulk'):
    #   parameters['control_file']='/home/kamal/inelast.mod'
    # else:
    #   parameters['control_file']='/home/kamal/inelast_nobox.mod'

    folder = str(os.getcwd()) + str("/") + str(jobname)
    if not os.path.exists(folder):
        os.makedirs(str(jobname))
    os.chdir(str(jobname))
    print("folder name", folder)
    forces = "na"
    if os.path.isfile("./log.lammps"):
        try:

            (
                en,
                press,
                toten,
                c11,
                c22,
                c33,
                c12,
                c13,
                c23,
                c44,
                c55,
                c66,
                c14,
                c16,
                c24,
                c25,
                c26,
                c34,
                c35,
                c36,
                c45,
                c46,
                c56,
            ) = analyz_loge("./log.lammps")
        except:
            pass
    else:
        # pass
        # line=str("#PBS -l pmem=")+str(mem)+'\n'
        # f.write(line)
        write_lammps_data(structure=mat.structure, file="data")
        write_lammps_in(
            structure=mat.structure,
            lammps_in="init.mod",
            lammps_in1="potential.mod",
            lammps_in2="in.main",
            parameters=parameters,
        )
        if "elast" in parameters["control_file"]:
            import jarvis

            base_path = jarvis.__file__.split("/__init__.py")[0]
            displace_file = os.path.join(
                base_path, "lammps", "module_files", "displace.mod"
            )
            shutil.copy2(displace_file, "./")
        os.system(parameters["exec"])
        initial_str = read_data(data="data", ff="potential.mod")

        ####time.sleep(100)
        (
            en,
            press,
            toten,
            c11,
            c22,
            c33,
            c12,
            c13,
            c23,
            c44,
            c55,
            c66,
            c14,
            c16,
            c24,
            c25,
            c26,
            c34,
            c35,
            c36,
            c45,
            c46,
            c56,
        ) = analyz_loge("./log.lammps")

    print("initial,final sr", os.getcwd())
    initial_str = read_data(data="data", ff="potential.mod")
    final_str = read_data(data="data0", ff="potential.mod")
    try:
        forces = read_dump(data="0.dump", ff="potential.mod")
    except:
        pass
    print("initial,final sr2", os.getcwd())

    data_cal = []
    data_cal.append(
        {
            "jobname": jobname,
            "poscar": mat.as_dict(),
            "initial_pos": initial_str.as_dict(),
            "pair_style": str(parameters["pair_style"]),
            "pair_coeff": str(parameters["pair_coeff"]),
            "final_energy": float(toten),
            "en": en,
            "press": press,
            "final_str": final_str.as_dict(),
        }
    )
    json_file = str(jobname) + str(".json")
    os.chdir("../")
    f_json = open(json_file, "w")
    f_json.write(json.dumps(data_cal, indent=4, cls=MontyEncoder))
    f_json.close()
    return en, final_str, forces


if __name__ == "__main__":
    import jarvis

    base_path = jarvis.__file__.split("/__init__.py")[0]
    pos = os.path.join(
        base_path,
        "lammps",
        "examples",
        "Al03.eam.alloy_nist",
        "bulk@mp-134_fold",
        "POSCAR",
    )
    pair_coeff = os.path.join(base_path, "lammps", "examples", "Al03.eam.alloy")

    control_file = os.path.join(base_path, "lammps", "module_files", "inelast.mod")

    mat = Poscar.from_file(pos)

    parameters = {
        "c_size": 3,
        "exec": "/opt/lammps/bin/lammps  <in.main >out",
        "pair_style": "eam/alloy",
        "pair_coeff": pair_coeff,
        "atom_style": "charge",
        "control_file": control_file,
    }
    run_job(mat=mat, parameters=parameters)
    main_func(mat=mat, parameters=parameters)


from jarvis.spillage.spillage import *
import os

wf_so = os.path.join(os.path.dirname(__file__), "..", "spillage", "WAVECAR.soc")

wf_noso = os.path.join(os.path.dirname(__file__), "..", "spillage", "WAVECAR.nosoc")


def test_spillage():
    gamma_max, gamma_k, kmax, kpoints, noso_direct, so_direct, x, y, so_lumo, so_homo, noso_lumo, noso_homo = overlap_so_spinpol(
        wf_noso, wf_so
    )
    assert gamma_max == 1.3634111271008775



from pymatgen.symmetry.analyzer import SpacegroupAnalyzer
from pymatgen.electronic_structure.core import Spin, OrbitalType
from pymatgen.io.vasp.outputs import Oszicar, Outcar, Vasprun
from pymatgen.util.plotting import pretty_plot as get_publication_quality_plot

# from pymatgen.util.plotting_utils import get_publication_quality_plot
import matplotlib, yaml, os
from pymatgen.core.periodic_table import Element
from pymatgen.io.vasp.inputs import Potcar, Incar, Kpoints
from pymatgen.electronic_structure.plotter import (
    BSPlotterProjected,
    BSPlotter,
    DosPlotter,
)
import matplotlib

matplotlib.use("Agg")
import math, glob
from pymatgen.core.structure import Structure
import numpy as np
from pymatgen.analysis.elasticity.elastic import ElasticTensor


def bandgap(vrun="", occu_tol=0.001):
    """
    Parses vasprun.xml and gives bandgap 

    Args:
        vrun: path to vasprun.xml 
        occu_tol: occupation tolerance
    Returns:
           eg: bandgap
           direct: direct/indirect bandgap
    """
    v = Vasprun(vrun, occu_tol=occu_tol)
    eg = float(v.eigenvalue_band_properties[0])
    direct = v.eigenvalue_band_properties[3]

    return eg, direct


def bandstr(vrun="", kpfile="", filename=".", plot=False):
    """
    Plot electronic bandstructure
 
    Args:
        vrun: path to vasprun.xml
        kpfile:path to line mode KPOINTS file 
    Returns:
           matplotlib object
    """

    run = Vasprun(vrun, parse_projected_eigen=True)
    bands = run.get_band_structure(kpfile, line_mode=True, efermi=run.efermi)
    bsp = BSPlotter(bands)
    zero_to_efermi = True
    bandgap = str(round(bands.get_band_gap()["energy"], 3))
    # print "bg=",bandgap
    data = bsp.bs_plot_data(zero_to_efermi)
    plt = get_publication_quality_plot(12, 8)
    plt.close()
    plt.clf()
    band_linewidth = 3
    x_max = data["distances"][-1][-1]
    # print (x_max)
    for d in range(len(data["distances"])):
        for i in range(bsp._nb_bands):
            plt.plot(
                data["distances"][d],
                [
                    data["energy"][d]["1"][i][j]
                    for j in range(len(data["distances"][d]))
                ],
                "b-",
                linewidth=band_linewidth,
            )
            if bsp._bs.is_spin_polarized:
                plt.plot(
                    data["distances"][d],
                    [
                        data["energy"][d]["-1"][i][j]
                        for j in range(len(data["distances"][d]))
                    ],
                    "r--",
                    linewidth=band_linewidth,
                )
    bsp._maketicks(plt)
    if bsp._bs.is_metal():
        e_min = -10
        e_max = 10
        band_linewidth = 3

    for cbm in data["cbm"]:
        plt.scatter(cbm[0], cbm[1], color="r", marker="o", s=100)

        for vbm in data["vbm"]:
            plt.scatter(vbm[0], vbm[1], color="g", marker="o", s=100)

    plt.xlabel(r"$\mathrm{Wave\ Vector}$", fontsize=30)
    ylabel = (
        r"$\mathrm{E\ -\ E_f\ (eV)}$" if zero_to_efermi else r"$\mathrm{Energy\ (eV)}$"
    )
    plt.ylabel(ylabel, fontsize=30)
    plt.ylim(-4, 4)
    plt.xlim(0, x_max)
    plt.tight_layout()
    if plot == True:
        plt.savefig(filename, img_format="png")
        plt.close()

    return plt


def plot_dos(vrun="", low_lim=-5, up_lim=10, filename=".", extra="", plot=False):
    """
    Plot total density of states

    Args:
       vrun: vasprun.xml path
       low_lim: lower energy limit in eV
       up_lim: upper energy limit in eV
    Returns:
          matplotlib objects for
          total, orbital and element project density of states
    """

    run = Vasprun(vrun, occu_tol=1e-2)
    complete_dos = run.complete_dos
    totup = complete_dos.get_densities(spin=Spin.up)
    totdn = complete_dos.get_densities(spin=Spin.down)
    en = complete_dos.energies
    bandgap = str(run.eigenvalue_band_properties[0])
    ef = complete_dos.efermi
    en[:] = [x - ef for x in en]

    plt1 = get_publication_quality_plot(14, 10)
    # plt1.close()
    # plt1.clf()
    plt1.xlabel("Energies (eV)")
    plt1.ylabel("Tot. DOS (arb. unit.)")
    plt1.plot(en, totup, "b", linewidth=3)
    plt1.plot(en, -totdn, "r", linewidth=3)
    plt1.xlim(low_lim, up_lim)
    plt1.tight_layout()
    if plot == True:
        tmp = str(filename) + str("/TDos") + str(extra) + str(".png")
        plt1.savefig(tmp)

    spd_dos = complete_dos.get_spd_dos()
    sdosup = spd_dos[OrbitalType.s].densities[Spin.up]
    pdosup = spd_dos[OrbitalType.p].densities[Spin.up]
    ddosup = spd_dos[OrbitalType.d].densities[Spin.up]
    sdosdn = spd_dos[OrbitalType.s].densities[Spin.down]
    pdosdn = spd_dos[OrbitalType.p].densities[Spin.down]
    ddosdn = spd_dos[OrbitalType.d].densities[Spin.down]

    plt2 = get_publication_quality_plot(14, 10)
    # plt2.close()
    # plt2.clf()
    plt2.plot(en, sdosup, "r", linewidth=3, label="s")
    plt2.plot(en, -sdosdn, "r", linewidth=3)
    plt2.plot(en, pdosup, "g", linewidth=3, label="p")
    plt2.plot(en, -pdosdn, "g", linewidth=3)
    plt2.plot(en, ddosup, "b", linewidth=3, label="d")
    plt2.plot(en, -ddosdn, "b", linewidth=3)
    plt2.xlabel("Energies (eV)")
    plt2.ylabel("Orb. DOS (arb. unit.)")
    plt2.legend(prop={"size": 26})
    plt2.xlim(low_lim, up_lim)
    plt2.tight_layout()
    if plot == True:
        tmp = str(filename) + str("/Dos") + str(extra) + str(".png")
        plt2.savefig(tmp)

    plt3 = get_publication_quality_plot(14, 10)
    # plt3.close()
    # plt3.clf()
    elt_dos = complete_dos.get_element_dos()
    at_symbs = set(run.atomic_symbols)
    for i, sym in enumerate(at_symbs):
        elt = elt_dos[Element(sym)]
        # color = cmap(float(i)/len(at_symbs))
        if i == 0:
            color = "b"
        elif i == 1:
            color = "g"
        elif i == 2:
            color = "r"
        elif i == 3:
            color = "c"
        elif i == 4:
            color = "m"
        elif i == 5:
            color = "y"
        elif i == 6:
            color = "k"
        else:
            color = "k"
            print("Use different color scheme")

        plt3.plot(en, elt.densities[Spin.up], color=color, linewidth=3, label=sym)
        plt3.plot(en, -elt.densities[Spin.down], color=color, linewidth=3)
        plt3.xlabel("Energies (eV)")
        plt3.ylabel("Elm. DOS (arb. unit.)")
        plt3.legend(prop={"size": 26})
        plt3.xlim(low_lim, up_lim)
        plt3.tight_layout()
    if plot == True:
        tmp = str(filename) + str("/EDos") + str(extra) + str(".png")
        plt3.savefig(tmp)

    return plt1, plt2, plt3


def plot_enc_convergence(
    directory="../vasp/examples/SiOptb88/", plot=False, filename="."
):
    """
    Plot convergence for plane-wave cut-off data
    Works only if jobs run through jarvis-tools framework
    
    Args:
       directory: parent directory for job run
    Returns:
           matplotlib object, converged cut-off value
    """

    x = []
    y = []
    for a in glob.glob(str(directory) + str("/*.json")):
        if "MAIN-RELAX" in a:
            main_inc = str(a.split(".json")[0]) + str("/") + str("INCAR")
            main_inc_obj = Incar.from_file(main_inc)
            convg_encut = float(main_inc_obj["ENCUT"])
        elif "ENCUT" in a:
            run = str(a.split(".json")[0]) + str("/") + str("vasprun.xml")
            contcar = Structure.from_file(
                (a.split(".json")[0]) + str("/") + str("CONTCAR")
            )
            vrun = Vasprun(run)
            infile = str(a.split(".json")[0]) + str("/") + str("INCAR")
            inc = Incar.from_file(infile)
            encut = inc["ENCUT"]
            en = float(vrun.final_energy)  # /float(contcar.composition.num_atoms)
            x.append(encut)
            y.append(en)
    order = np.argsort(x)
    xs = np.array(x)[order]
    ys = np.array(y)[order]
    plt = get_publication_quality_plot(14, 10)
    plt.ylabel("Energy (eV)")
    plt.plot(xs, ys, "s-", linewidth=2, markersize=10)
    plt.xlabel("Increment in ENCUT ")
    ax = plt.gca()
    ax.get_yaxis().get_major_formatter().set_useOffset(False)
    plt.title(str("Converged at ") + str(int(convg_encut)) + str("eV"), fontsize=26)
    # filename=str('Encut.png')
    plt.tight_layout()
    if plot == True:
        plt.savefig(filename)
        plt.close()

    return plt, convg_encut


def plot_kp_convergence(
    directory="../vasp/examples/SiOptb88/", plot=False, filename="."
):
    """
    Plot convergence for k-points data
    Works only if jobs run through jarvis-tools framework
    
    Args:
       directory: parent directory for job run
    Returns:
           matplotlib object, converged k-points value
    """

    x = []
    y = []
    for a in glob.glob(str(directory) + str("/*.json")):
        if "MAIN-RELAX" in a:
            main_kp = str(a.split(".json")[0]) + str("/") + str("KPOINTS")
            main_kp_obj = Kpoints.from_file(main_kp)
            [k1, k2, k3] = main_kp_obj.kpts[0]
        elif "KPOINT" in a:
            k = int(float(str(a.split("-")[-1]).split(".json")[0]))
            run = str(a.split(".json")[0]) + str("/") + str("vasprun.xml")
            vrun = Vasprun(run)

            kpfile = str(a.split(".json")[0]) + str("/") + str("KPOINTS")
            contcar = Structure.from_file(
                (a.split(".json")[0]) + str("/") + str("CONTCAR")
            )
            kpoints = Kpoints.from_file(kpfile)
            [xx, yy, zz] = kpoints.kpts[0]
            en = float(vrun.final_energy)  # /float(contcar.composition.num_atoms)
            # en =float(vrun.final_energy)/float(contcar.composition.num_atoms)
            # print "ENERGYYY,AT",en,float(contcar.composition.num_atoms)
            x.append(k)
            y.append(en)

    order = np.argsort(x)
    xs = np.array(x)[order]
    ys = np.array(y)[order]
    len_xs = len(xs)
    xs1 = []
    ys1 = []
    target_ys = ys[-1]
    # print "target=",target_ys
    for i, el in enumerate(ys):
        if el <= (float(target_ys) + 0.002):
            xs1.append(xs[i])
            ys1.append(ys[i])
    # print "xs,ys=",xs,ys
    # print "xs1,ys1=",xs1,ys1
    left, bottom, width, height = [0.5, 0.5, 0.35, 0.35]
    plt = get_publication_quality_plot(14, 10)
    fig, ax1 = plt.subplots()
    plt.xlabel("Increment in K point", fontsize=20)
    plt.ylabel("Energy (eV)", fontsize=20)
    plt.title(
        str("Converged at ")
        + str(k1)
        + str("x")
        + str(k2)
        + str("x")
        + str(k3)
        + str(" ")
        + str("Automatic Mesh ")
        + str(max(x) - 25),
        fontsize=14,
    )
    plt.xticks(fontsize=20)
    plt.yticks(fontsize=20)
    ax2 = fig.add_axes([left, bottom, width, height])
    # ax2.plot(xs1,ys1, '.-',linewidth=2,markersize=10)
    ax1.plot(xs, ys, "s-", linewidth=2, markersize=10)
    el_list = sorted(list(contcar.symbol_set))
    search = "-".join([item for item in el_list])
    # ax1.xlabel('Increment in K point')
    # print "xs,ys"
    # print xs
    # print ys
    plt.plot(xs1, ys1, ".-", linewidth=2, markersize=10)
    plt.ylim([float(target_ys) + 0.002, float(target_ys) - 0.002])
    ax = plt.gca()
    ax.get_yaxis().get_major_formatter().set_useOffset(False)
    filename = str("KDen.png")
    k_convg = str(xx) + str("x") + str(yy) + str("x") + str(zz)
    plt.tight_layout()
    if plot == True:
        plt.savefig(filename)
        plt.close()
    return plt, k_convg


def IP_optics(vrun="", filename=".", extra="", plot=False):
    """
    Optoelectronic properties derived from dielectric function 

    See http://www.wien2k.at/reg_user/textbooks/WIEN2k_lecture-notes_2013/optic_handout.pdf
    http://www.phys.ufl.edu/~tanner/Wooten-OpticalPropertiesOfSolids-2up.pdf

    Args:
       vrun: path to vasprun.xml
    Reurns:
          info: dictionary with optoelectronic information
          energy: energy bins generally 0 to 40 eV
          real_part: real part of dielectric function
          imag_part: imaginary part of dielectric function
          absorption: absorption coefficient in cm^-1
       
    TODO: shorten the code with multi-dimensional np array   
    """

    info = {}

    storedir = filename

    run = Vasprun(vrun, occu_tol=1e-2)
    erange = len(run.dielectric[0])
    en = []
    realx = []
    imagx = []
    absorpx = []
    refrx = []
    reflx = []
    eelsx = []
    extcx = []
    opt_conx = []
    realy = []
    imagy = []
    absorpy = []
    refry = []
    refly = []
    eelsy = []
    extcy = []
    opt_cony = []
    realz = []
    imagz = []
    absorpz = []
    refrz = []
    reflz = []
    eelsz = []
    extcz = []
    opt_conz = []
    H = 4.13566733 * (10 ** (-15))
    # c0=2.99792458
    c0 = 2.99792458 * (math.pow(10, 8))
    for i in range(0, erange - 1):
        en.append(run.dielectric[0][i])
        realx.append(run.dielectric[1][i][0])
        imagx.append(run.dielectric[2][i][0])
        ab_valx = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][0]
                        + math.sqrt(
                            (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                            + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpx.append(ab_valx)
        refr_valx = float(
            math.sqrt(
                (run.dielectric[1][i][0])
                + math.sqrt(
                    (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                    + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                )
            )
        ) / float(1.4142)
        refrx.append(refr_valx)
        eels_valx = float(run.dielectric[2][i][0]) / float(
            (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
            + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
        )
        eelsx.append(eels_valx)
        extc_valx = float(
            math.sqrt(
                -(run.dielectric[1][i][0])
                + math.sqrt(
                    (run.dielectric[2][i][0]) * (run.dielectric[2][i][0])
                    + (run.dielectric[1][i][0]) * (run.dielectric[1][i][0])
                )
            )
        ) / float(1.4142)
        extcx.append(extc_valx)
        refl_valx = ((refr_valx - 1) * (refr_valx - 1) + extc_valx * extc_valx) / (
            (refr_valx + 1) * (refr_valx + 1) + extc_valx * extc_valx
        )
        reflx.append(refl_valx)
        opt_valx = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][0])
        )
        opt_conx.append(opt_valx)

        realy.append(run.dielectric[1][i][1])
        imagy.append(run.dielectric[2][i][1])

        ab_valy = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][1]
                        + math.sqrt(
                            (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                            + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpy.append(ab_valy)
        refr_valy = float(
            math.sqrt(
                (run.dielectric[1][i][1])
                + math.sqrt(
                    (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                    + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                )
            )
        ) / float(1.4142)
        refry.append(refr_valy)
        eels_valy = float(run.dielectric[2][i][1]) / float(
            (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
            + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
        )
        eelsy.append(eels_valy)
        extc_valy = float(
            math.sqrt(
                -(run.dielectric[1][i][1])
                + math.sqrt(
                    (run.dielectric[2][i][1]) * (run.dielectric[2][i][1])
                    + (run.dielectric[1][i][1]) * (run.dielectric[1][i][1])
                )
            )
        ) / float(1.4142)
        extcy.append(extc_valy)
        refl_valy = ((refr_valy - 1) * (refr_valy - 1) + extc_valy * extc_valy) / (
            (refr_valy + 1) * (refr_valy + 1) + extc_valy * extc_valy
        )
        refly.append(refl_valy)
        opt_valy = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][1])
        )
        opt_cony.append(opt_valy)

        realz.append(run.dielectric[1][i][2])
        imagz.append(run.dielectric[2][i][2])

        ab_valz = 1.4142 * (
            (float(run.dielectric[0][i]) / float(H))
            * (
                float(
                    math.sqrt(
                        -run.dielectric[1][i][2]
                        + math.sqrt(
                            (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                            + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                        )
                    )
                )
                / float(float(c0) * 100.0)
            )
        )
        absorpz.append(ab_valz)
        refr_valz = float(
            math.sqrt(
                (run.dielectric[1][i][2])
                + math.sqrt(
                    (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                    + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                )
            )
        ) / float(1.4142)
        refrz.append(refr_valz)
        eels_valz = float(run.dielectric[2][i][2]) / float(
            (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
            + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
        )
        eelsz.append(eels_valz)
        extc_valz = float(
            math.sqrt(
                -(run.dielectric[1][i][2])
                + math.sqrt(
                    (run.dielectric[2][i][2]) * (run.dielectric[2][i][2])
                    + (run.dielectric[1][i][2]) * (run.dielectric[1][i][2])
                )
            )
        ) / float(1.4142)
        extcz.append(extc_valz)
        refl_valz = ((refr_valz - 1) * (refr_valz - 1) + extc_valz * extc_valz) / (
            (refr_valz + 1) * (refr_valz + 1) + extc_valz * extc_valz
        )
        reflz.append(refl_valz)
        opt_valz = (
            float(float(run.dielectric[0][i]) / float(H))
            / float(4 * math.pi)
            * (run.dielectric[2][i][2])
        )
        opt_conz.append(opt_valz)

    if plot == True:
        # plt.close()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, realx, linewidth=2, label=r"$\epsilon_1x$")
        plt.plot(en, realy, linewidth=2, label=r"$\epsilon_1y$")
        plt.plot(en, realz, linewidth=2, label=r"$\epsilon_1z$")
        plt.legend(prop={"size": 26})
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Dielec. function (Real part)")
        filename = str(storedir) + str("/") + str("Real") + str(extra) + str(".png")
        plt.tight_layout()
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, imagx, linewidth=2, label=r"$\epsilon_2x$")
        plt.plot(en, imagy, linewidth=2, label=r"$\epsilon_2y$")
        plt.plot(en, imagz, linewidth=2, label=r"$\epsilon_2z$")
        plt.xlabel("Energy (eV)")
        plt.xlim([0, 30])
        plt.ylabel("Dielec. function(Imag part)")
        filename = str(storedir) + str("/") + str("Imag") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, refrx, linewidth=2, label=r"$n_x$")
        plt.plot(en, refry, linewidth=2, label=r"$n_y$")
        plt.plot(en, refrz, linewidth=2, label=r"$n_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Refractive index")
        filename = str(storedir) + str("/") + str("Refr") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, extcx, linewidth=2, label=r"$k_x$")
        plt.plot(en, extcy, linewidth=2, label=r"$k_y$")
        plt.plot(en, extcz, linewidth=2, label=r"$k_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Extinction Coefficient")
        filename = str(storedir) + str("/") + str("Extc") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, absorpx, linewidth=2, label=r"$\alpha_x$")
        plt.plot(en, absorpy, linewidth=2, label=r"$\alpha_y$")
        plt.plot(en, absorpz, linewidth=2, label=r"$\alpha_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Absorption coefficient")
        filename = str(storedir) + str("/") + str("Absorp") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, eelsx, linewidth=2, label=r"$e_x$")
        plt.plot(en, eelsy, linewidth=2, label=r"$e_y$")
        plt.plot(en, eelsz, linewidth=2, label=r"$e_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Energy loss spectrum")
        filename = str(storedir) + str("/") + str("ELS") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

        plt.clf()
        plt = get_publication_quality_plot(14, 10)
        plt.plot(en, opt_conx, linewidth=2, label=r"$\sigma_x$")
        plt.plot(en, opt_cony, linewidth=2, label=r"$\sigma_y$")
        plt.plot(en, opt_conz, linewidth=2, label=r"$\sigma_z$")
        plt.xlim([0, 30])
        plt.xlabel("Energy (eV)")
        plt.ylabel("Optical conductivity (Real part)")
        filename = str(storedir) + str("/") + str("Opt_con") + str(extra) + str(".png")
        plt.tight_layout()
        plt.legend(prop={"size": 26})
        plt.savefig(filename)
        plt.tight_layout()
        plt.close()

    info["absorption"] = [absorpx, absorpy, absorpz]
    info["energy"] = en
    info["real_part"] = [realx, realy, realz]
    info["imag_part"] = [imagx, imagy, imagz]
    info["refr_index"] = [refrx, refry, refrz]
    info["extinc"] = [extcx, extcy, extcz]
    info["reflectance"] = [reflx, refly, reflz]
    info["EELS"] = [eelsx, eelsy, eelsz]
    info["opt_conduct"] = [opt_conx, opt_cony, opt_conz]
    print(info["real_part"][0][0])
    return info


def elastic_props(outcar="", vacuum=False):
    """
    Obtain elastic tensor and calculate related properties

    Args:
        outcar: OUTCAR file path
        vacuum: whether the structure has vaccum such as 2D materials
        for vacuum structures bulk and shear mod. needs extra attenstion
        and elastic tensor are in Nm^-1 rather than GPa
    Returns:
          info: data for elastic tensor (in string and object representation), bulk, shear modulus, and phonon modes
    """
    if vacuum == True:
        contcar = Structure.from_file(str(outcar).replace("OUTCAR", "POSCAR"))
        ratio_c = 0.1 * float(
            abs(contcar.lattice.matrix[2][2])
        )  # *(10**9)*(10**-10) #N/m unit
    el_tens = "na"
    KV = "na"
    GV = "na"
    spin = "na"
    info = {}
    ratio_c = 1.0
    v = open(outcar, "r")
    lines = v.read().splitlines()
    for i, line in enumerate(lines):
        if "TOTAL ELASTIC MODULI (kBar)" in line:
            c11 = lines[i + 3].split()[1]
            c12 = lines[i + 3].split()[2]
            c13 = lines[i + 3].split()[3]
            c14 = lines[i + 3].split()[4]
            c15 = lines[i + 3].split()[5]
            c16 = lines[i + 3].split()[6]
            c21 = lines[i + 4].split()[1]
            c22 = lines[i + 4].split()[2]
            c23 = lines[i + 4].split()[3]
            c24 = lines[i + 4].split()[4]
            c25 = lines[i + 4].split()[5]
            c26 = lines[i + 4].split()[6]
            c31 = lines[i + 5].split()[1]
            c32 = lines[i + 5].split()[2]
            c33 = lines[i + 5].split()[3]
            c34 = lines[i + 5].split()[4]
            c35 = lines[i + 5].split()[5]
            c36 = lines[i + 5].split()[6]
            c41 = lines[i + 6].split()[1]
            c42 = lines[i + 6].split()[2]
            c43 = lines[i + 6].split()[3]
            c44 = lines[i + 6].split()[4]
            c45 = lines[i + 6].split()[5]
            c46 = lines[i + 6].split()[6]
            c51 = lines[i + 7].split()[1]
            c52 = lines[i + 7].split()[2]
            c53 = lines[i + 7].split()[3]
            c54 = lines[i + 7].split()[4]
            c55 = lines[i + 7].split()[5]
            c56 = lines[i + 7].split()[6]
            c61 = lines[i + 8].split()[1]
            c62 = lines[i + 8].split()[2]
            c63 = lines[i + 8].split()[3]
            c64 = lines[i + 8].split()[4]
            c65 = lines[i + 8].split()[5]
            c66 = lines[i + 8].split()[6]
            c11 = round(ratio_c * float(c11) / float(10), 1)
            c12 = round(ratio_c * float(c12) / float(10), 1)
            c13 = round(ratio_c * float(c13) / float(10), 1)
            c14 = round(ratio_c * float(c14) / float(10), 1)
            c15 = round(ratio_c * float(c15) / float(10), 1)
            c16 = round(ratio_c * float(c16) / float(10), 1)
            c21 = round(ratio_c * float(c21) / float(10), 1)
            c22 = round(ratio_c * float(c22) / float(10), 1)
            c23 = round(ratio_c * float(c23) / float(10), 1)
            c24 = round(ratio_c * float(c24) / float(10), 1)
            c25 = round(ratio_c * float(c25) / float(10), 1)
            c26 = round(ratio_c * float(c26) / float(10), 1)
            c31 = round(float(c31) / float(10), 1)
            c32 = round(float(c32) / float(10), 1)
            c33 = round(float(c33) / float(10), 1)
            c34 = round(float(c34) / float(10), 1)
            c35 = round(float(c35) / float(10), 1)
            c36 = round(float(c36) / float(10), 1)
            c41 = round(float(c41) / float(10), 1)
            c42 = round(float(c42) / float(10), 1)
            c43 = round(float(c43) / float(10), 1)
            c44 = round(float(c44) / float(10), 1)
            c45 = round(float(c45) / float(10), 1)
            c46 = round(float(c46) / float(10), 1)
            c51 = round(float(c51) / float(10), 1)
            c52 = round(float(c52) / float(10), 1)
            c53 = round(float(c53) / float(10), 1)
            c54 = round(float(c54) / float(10), 1)
            c55 = round(float(c55) / float(10), 1)
            c56 = round(float(c56) / float(10), 1)
            c61 = round(float(c61) / float(10), 1)
            c62 = round(float(c62) / float(10), 1)
            c63 = round(float(c63) / float(10), 1)
            c64 = round(float(c64) / float(10), 1)
            c65 = round(float(c65) / float(10), 1)
            c66 = round(float(c66) / float(10), 1)
            KV = float((c11 + c22 + c33) + 2 * (c12 + c23 + c31)) / float(9)
            GV = float(
                (c11 + c22 + c33) - (c12 + c23 + c31) + 3 * (c44 + c55 + c66)
            ) / float(15)
            KV = round(KV, 3)
            GV = round(GV, 3)
            break
    v.close()

    # Convenient string representation for storage

    el_tens = (
        str(c11)
        + str(",")
        + str(c12)
        + str(",")
        + str(c13)
        + str(",")
        + str(c14)
        + str(",")
        + str(c15)
        + str(",")
        + str(c16)
        + str(",")
        + str(c21)
        + str(",")
        + str(c22)
        + str(",")
        + str(c23)
        + str(",")
        + str(c24)
        + str(",")
        + str(c25)
        + str(",")
        + str(c26)
        + str(",")
        + str(c31)
        + str(",")
        + str(c32)
        + str(",")
        + str(c33)
        + str(",")
        + str(c34)
        + str(",")
        + str(c35)
        + str(",")
        + str(c36)
        + str(",")
        + str(c41)
        + str(",")
        + str(c42)
        + str(",")
        + str(c43)
        + str(",")
        + str(c44)
        + str(",")
        + str(c45)
        + str(",")
        + str(c46)
        + str(",")
        + str(c51)
        + str(",")
        + str(c52)
        + str(",")
        + str(c53)
        + str(",")
        + str(c54)
        + str(",")
        + str(c55)
        + str(",")
        + str(c56)
        + str(",")
        + str(c61)
        + str(",")
        + str(c62)
        + str(",")
        + str(c63)
        + str(",")
        + str(c64)
        + str(",")
        + str(c65)
        + str(",")
        + str(c66)
    )

    cij = np.empty((6, 6), dtype=float)
    elast = np.array(el_tens.split(","), dtype="float")
    count = 0
    for ii in range(6):
        for jj in range(6):
            cij[ii][jj] = elast[count]
    el_tens2 = ElasticTensor.from_voigt(cij)

    modes = []
    try:
        for i in lines:
            if "cm-1" in i and "meV" in i:
                mod = float(i.split()[7])
                if mod not in modes:
                    modes.append(float(mod))
    except:
        pass

    info["el_tens_str"] = el_tens
    info["el_tens_obj"] = el_tens2
    info["KV"] = KV
    info["GV"] = GV
    info["modes"] = modes

    return info


def magnetic_moment(oszicar=""):
    """
    Obtain total orbital magnetic moment
    Magnetic moment info is found in both OSZICAR and OUTCAR
    We prefer OSZICAR values

    Args:
        oszicar: path to OSZICAR
    Returns:
          magmom: orbital magnetic moment in Bohr-magneton
    """
    magmom = Oszicar(oszicar).ionic_steps[-1]["mag"]
    # magmom_out=out.total_mag #for outcar
    return magmom


def get_area(strt=""):
    """
    Get area of a Structure object along z-direction

    Args: 
        strt: Structure object
    Returns:
           area
    """

    m = strt.lattice.matrix
    area = np.linalg.norm(np.cross(m[0], m[1]))
    return area


def get_spacegroup(strt=""):
    """
    Get spacegroup of a Structure pbject
    
    Args:
        strt: Structure object
    Returns:
           num: spacegroup number
           symb: spacegroup symbol
    """
    finder = SpacegroupAnalyzer(strt)
    num = finder.get_space_group_number()
    symb = finder.get_space_group_symbol()
    return num, symb


# vrun='../vasp/examples/SiOptb88/MAIN-OPTICS-bulk@mp_149/vasprun.xml'
# out='../vasp/examples/SiOptb88/MAIN-ELASTIC-bulk@mp_149/OUTCAR'
# osz='../vasp/examples/SiOptb88/MAIN-RELAX-bulk@mp_149/OSZICAR'
# m=magnetic_moment(osz)
# print (m)
# info=elastic_props(out)
# print (info['KV'])
# op=IP_optics(vrun,plot=True)
##op['real_part'][0][0]
# print (op)
# plot_kp_convergence()



from jarvis.spillage.vaspwfc import vaspwfc
import os

wf_so = os.path.join(os.path.dirname(__file__), "..", "spillage", "WAVECAR.soc")

def test_vaspwfc():
    so = vaspwfc(fnm=wf_so, lsorbit=True)
    so.printWF()
    so_k, so_bands = so.readWFBand()
    print (so_bands)



"""
Skorch with JARVIS-ML

From https://github.com/skorch-dev/skorch/blob/efeead6f1c361a4c12d35e78dd84eb7fe1c5232e/notebooks/Basic_Usage.ipynb
"""
from skorch import NeuralNetRegressor
import numpy as np
from sklearn.model_selection import train_test_split
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler
from sklearn.feature_selection import VarianceThreshold
from sklearn.pipeline import Pipeline


class RegressorModule(nn.Module):
    def __init__(self, num_units=10, nonlin=F.relu):
        super(RegressorModule, self).__init__()
        self.num_units = num_units
        self.nonlin = nonlin

        self.dense0 = nn.Linear(20, num_units)
        self.nonlin = nonlin
        self.dense1 = nn.Linear(num_units, 10)
        self.output = nn.Linear(10, 1)

    def forward(self, X, **kwargs):
        X = self.nonlin(self.dense0(X))
        X = F.relu(self.dense1(X))
        X = self.output(X)
        return X


def simple_regression(X=[], Y=[], plot=False, preprocess=True):
    """
  Quickly train simple regression models without hyperparameter-optimization
  Args:
      X: input features
      Y: Target data
      plot: whether to make a parity plot with ML models
      preprocess: whether to apply standard preprocessing techniques
  Returns:
  """

    net_regr = NeuralNetRegressor(
        RegressorModule,
        max_epochs=20,
        lr=0.1,
        #     device='cuda',  # uncomment this to train with CUDA
    )
    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        X, Y, jid, random_state=1, test_size=0.1
    )
    pipe = Pipeline(
        [
            ("stdscal", StandardScaler()),
            ("vart", VarianceThreshold(1e-4)),
            ("est", net_regr),
        ]
    )
    if preprocess == True:
        model = pipe
    else:
        model = net_regr
    model.fit(X_train, y_train)
    pred = model.predict(X_test)
    reg_sc = regr_scores(y_test, pred)
    if plot == True:
        plt.plot(reg_sc["pred"], reg_sc["test"], ".", label=str(type(i).__name__)[0:4])
    print(type(i).__name__, round(reg_sc["mae"], 3), round(reg_sc["rmse"], 3))
    if plot == True:
        plt.legend()
        plt.xlabel("DFT")
        plt.ylabel("ML")


if __name__ == "__main__":
    X, Y, jid = jdata(prop="exfoliation_en")
    simple_regression(x=X, y=Y)



import matplotlib.pyplot as plt

plt.switch_backend("agg")
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.model_selection import cross_val_score
from sklearn.model_selection import KFold
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import MinMaxScaler
from tensorflow.python.keras.models import Sequential
from tensorflow.python.keras.layers import Dense
from tensorflow.python.keras.wrappers.scikit_learn import KerasRegressor
from jarvis.db.static.explore_db import get_3d_dataset, get_2d_dataset, get_ml_dataset


def simple_regression(x=[], y=[], plot=False):
    y = np.reshape(y, (-1, 1))
    scaler_x = MinMaxScaler()
    scaler_y = MinMaxScaler()
    print(scaler_x.fit(x))
    xscale = scaler_x.transform(x)
    print(scaler_y.fit(y))
    yscale = scaler_y.transform(y)

    X_train, X_test, y_train, y_test = train_test_split(xscale, yscale)
    model = Sequential()
    model.add(Dense(12, input_dim=1557, kernel_initializer="normal", activation="relu"))
    model.add(Dense(8, activation="relu"))
    model.add(Dense(1, activation="linear"))
    model.summary()
    model.compile(loss="mse", optimizer="adam", metrics=["mse", "mae"])
    history = model.fit(
        X_train, y_train, epochs=150, batch_size=50, verbose=1, validation_split=0.2
    )
    print(history.history.keys())
    # "Loss"
    if plot == True:
        plt.plot(history.history["loss"])
        plt.plot(history.history["val_loss"])
        plt.title("model loss")
        plt.ylabel("loss")
        plt.xlabel("epoch")
        plt.legend(["train", "validation"], loc="upper left")
        plt.show()
    Xnew = x[-1]
    Xnew = scaler_x.transform(Xnew)
    ynew = model.predict(Xnew)
    ynew = scaler_y.inverse_transform(ynew)
    Xnew = scaler_x.inverse_transform(Xnew)
    print("X=%s, Predicted=%s" % (Xnew[0], ynew[0]))


if __name__ == "__main__":
    X, Y, jid = jdata(prop="exfoliation_en")
    simple_regression(x=X, y=Y)



from jarvis.sklearn.get_desc import get_comp_descp
from monty.serialization import loadfn, MontyDecoder, dumpfn
import numpy as np
from sklearn.metrics import mean_absolute_error, r2_score, mean_squared_error

try:
    import lightgbm.LGBMRegressor as lgb
except:
    print("WARNING!!!!: LightGBM is not installed, using sklearn GBM")
    print("Will try to use sklearn instead, errors are possible")
    from sklearn.ensemble import GradientBoostingRegressor as lgb

import matplotlib.pyplot as plt
from jarvis.db.static.explore_db import get_ml_dataset

# import lightgbm as lgb
import matplotlib.pyplot as plt


plt.switch_backend("agg")
import pandas as pd
from sklearn.datasets import load_boston
from sklearn.model_selection import (
    train_test_split,
    learning_curve,
    cross_val_score,
    cross_val_predict,
    GridSearchCV,
    RandomizedSearchCV,
)
import scipy as sp
import time, os, json, pprint
from sklearn.feature_selection import (
    SelectKBest,
    f_classif,
    SelectFromModel,
    VarianceThreshold,
)
from sklearn.pipeline import Pipeline
from sklearn.preprocessing import StandardScaler

"""
Steps for building a regression model
Download descriptors and material data from the following link:
https://figshare.com/articles/JARVIS-ML-CFID-descriptors_and_material_properties/6870101 
You can also make descriptors using get_comp_descp
Link to the paper: https://journals.aps.org/prmaterials/abstract/10.1103/PhysRevMaterials.2.083801
Websites: https://www.ctcms.nist.gov/jarvisml, https://jarvis.nist.gov
"""


def isfloat(value):
    """
  Check if an argument is float
  
  Args:
      value: string/int/float
  Returns:
         True/False 
  """

    try:
        float(value)
        return True
    except:
        return False
        pass


def jdata(data_file="jarvisml_cfid.json", prop=""):
    """
  Get data and descriptors for a particular properties
 
  Args:
      data_file: JSON containing descriptors and target properties
      It can be better to save JSON rather tha converting materials
      to descriptors all the time
      prop: property to train such as formation energy, bandgap etc.
  Returns:
         Generic X, Y and identifiers 
         The IDs are used to detect which samples could be trained well and
         which couldn't

  """

    d3 = get_ml_dataset()

    X = []
    Y = []
    jid = []
    for ii, i in enumerate(d3):
        y = i[prop]
        if isfloat(y):
            y = float(y)
            x = i["desc"]
            if len(x) == 1557 and any(np.isnan(x) for x in x.flatten()) == False:
                if "eps" in prop:
                    y = np.sqrt(float(y))
                if "mag" in prop:
                    num = get_number_formula_unit(i["strt"])
                    y = float(abs(y)) / float(num)
                X.append(x)
                Y.append(y)
                jid.append(i["jid"])
    print("Prop=", prop, len(X), len(Y))
    X = np.array(X).astype(np.float64)
    Y = np.array(Y).astype(np.float64)
    return X, Y, jid


def plot_learning_curve(
    estimator=None,
    scoring="neg_mean_absolute_error",
    title="",
    X=None,
    y=None,
    ylim=None,
    cv=5,
    # n_jobs=-1, train_sizes=np.linspace(.1, 1.0, 10),fname='fig.png'):
    n_jobs=1,
    train_sizes=np.linspace(0.01, 1.0, 50),
    fname="fig.png",
):
    """
    Taken from scikit-learn, added ways to store results in JSON format

    """

    plt.figure()
    # fname='fig.png'
    plt.title(title)
    mem = {}
    if ylim is not None:
        plt.ylim(*ylim)
    plt.xlabel("Training examples")
    plt.ylabel("Score")
    train_sizes, train_scores, test_scores = learning_curve(
        estimator, X, y, cv=cv, n_jobs=n_jobs, train_sizes=train_sizes, scoring=scoring
    )
    train_scores_mean = np.mean(train_scores, axis=1)
    train_scores_std = np.std(train_scores, axis=1)
    test_scores_mean = np.mean(test_scores, axis=1)
    test_scores_std = np.std(test_scores, axis=1)
    plt.grid()
    mem["train_sizes"] = train_sizes
    mem["train_scores_mean"] = train_scores_mean
    mem["test_scores_mean"] = test_scores_mean
    mem["train_scores_std"] = train_scores_std
    mem["test_scores_std"] = test_scores_std

    plt.fill_between(
        train_sizes,
        train_scores_mean - train_scores_std,
        train_scores_mean + train_scores_std,
        alpha=0.1,
        color="r",
    )
    plt.fill_between(
        train_sizes,
        test_scores_mean - test_scores_std,
        test_scores_mean + test_scores_std,
        alpha=0.1,
        color="g",
    )
    plt.plot(train_sizes, train_scores_mean, "o-", color="r", label="Training score")
    plt.plot(
        train_sizes, test_scores_mean, "o-", color="g", label="Cross-validation score"
    )

    plt.legend(loc="best")
    fname = str(fname) + str("_learn_1.png")
    plt.show()
    plt.savefig(fname)
    plt.close()

    data = str(fname) + str("learn_data")
    f = open(data, "w")
    line = (
        str("train_sizes ")
        + str("train_scores_mean ")
        + str("train_scores_std ")
        + str("test_scores_mean ")
        + str("test_scores_std ")
        + "\n"
    )
    f.write(line)
    for i, j, k, l, m in zip(
        train_sizes,
        train_scores_mean,
        train_scores_std,
        test_scores_mean,
        test_scores_std,
    ):
        line = (
            str(i)
            + str(" ")
            + str(j)
            + str(" ")
            + str(k)
            + str(" ")
            + str(l)
            + str(" ")
            + str(m)
            + str("\n")
        )
        f.write(line)
    f.close()

    plt.figure()
    plt.title(title)
    if ylim is not None:
        plt.ylim(*ylim)
    plt.xlabel("Training examples")
    plt.ylabel("Score")
    plt.grid()
    plt.fill_between(
        train_sizes,
        test_scores_mean - test_scores_std,
        test_scores_mean + test_scores_std,
        alpha=0.1,
        color="g",
    )
    plt.plot(
        train_sizes, test_scores_mean, "o-", color="g", label="Cross-validation score"
    )
    plt.legend(loc="best")
    fname = str(fname) + str("_learn_2.png")
    plt.show()
    plt.savefig(fname)
    plt.close()
    return mem


def regr_scores(pred, test):

    """
   Generic regresion scores

   Args:
       pred: predicted values
       test: held data for testing
   Returns:
        info: with metrics
   """

    rmse = np.sqrt(mean_squared_error(test, pred))
    r2 = r2_score(test, pred)
    mae = mean_absolute_error(test, pred)
    info = {}
    info["mae"] = mae
    info["rmse"] = rmse
    info["r2"] = r2
    info["test"] = test
    info["pred"] = pred
    return info


def get_lgbm(train_x, val_x, train_y, val_y, cv, n_jobs, scoring):
    """
    Train a lightgbm model

    Args:
        train_x: samples used for trainiing
        val_x: validation set
        train_y: train targets
        val_y: validation targets
        cv: # of cross-validations
        n_jobs: for making the job parallel
        scoring: scoring function to use such as MAE
    Returns:
           Best estomator
    """

    # Get converged boosting iterations with high learning rate, MAE as the convergence crietria

    lgbm = lgb(
        n_estimators=1000,
        learning_rate=0.1,
        max_depth=5,
        num_leaves=100,
        objective="regression",
        # min_data_in_leaf=2,
        n_jobs=-1,
        verbose=-1,
    )

    lgbm.fit(
        train_x,
        train_y,
        eval_set=[(val_x, val_y)],
        eval_metric="mae",
        # eval_metric='l1',
        early_stopping_rounds=10,
    )
    num_iteration = lgbm.best_iteration_
    print("num_iteration", num_iteration)
    print("in randomsearch cv")
    # Generally thousands of randomized search for optimal parameters
    # learning rate and num_leaves are very important
    param_dist = {
        #'boosting_type': [ 'dart'],
        #'boosting_type': ['gbdt', 'dart', 'rf'],
        #'num_leaves': sp.stats.randint(2, 1001),
        #'subsample_for_bin': sp.stats.randint(10, 1001),
        #'min_split_gain': sp.stats.uniform(0, 5.0),
        #'min_child_weight': sp.stats.uniform(1e-6, 1e-2),
        #'reg_alpha': sp.stats.uniform(0, 1e-2),
        #'reg_lambda': sp.stats.uniform(0, 1e-2),
        #'tree_learner': ['data', 'feature', 'serial', 'voting' ],
        #'application': ['regression_l1', 'regression_l2', 'regression'],
        #'bagging_freq': sp.stats.randint(1, 11),
        #'bagging_fraction': sp.stats.uniform(.1, 0.9),
        #'feature_fraction': sp.stats.uniform(.1, 0.9),
        #'learning_rate': sp.stats.uniform(1e-3, 0.9),
        #'est__num_leaves': [2,8,16],
        #'est__min_data_in_leaf': [1,2,4],
        #'est__learning_rate': [0.005,0.01,0.1],
        #'est__max_depth': [1,3,5], #sp.stats.randint(1, 501),
        #'est__n_estimators': [num_iteration,2*num_iteration,5*num_iteration],#sp.stats.randint(100, 20001),
        #'gpu_use_dp': [True, False],
        #'est__num_leaves': sp.stats.randint(3, 1000),
        #'est__max_depth': sp.stats.randint(1, 5),
        "est__learning_rate": sp.stats.uniform(1e-3, 0.9)
    }

    lgbm = lgb(
        objective="regression",
        # device='gpu',
        n_estimators=num_iteration,
        n_jobs=n_jobs,
        verbose=-1,
    )
    pipe = Pipeline(
        [
            ("stdscal", StandardScaler()),
            ("vart", VarianceThreshold(1e-4)),
            ("est", lgbm),
        ]
    )

    n_iter = 10
    # Increase n_iter
    rscv = RandomizedSearchCV(
        estimator=pipe,
        param_distributions=param_dist,
        cv=cv,
        scoring=scoring,
        n_iter=n_iter,
        n_jobs=n_jobs,
        verbose=3,
        refit=True,
    )
    rscv = rscv.fit(train_x, train_y)
    return rscv.best_estimator_


# Main-function
def run(
    version="version_1",
    scoring="neg_mean_absolute_error",
    cv=5,
    n_jobs=1,
    prop="op_gap",
    do_cv=False,
):
    """
 Generic run function to train-test split,
 find optimum number of boosting operations,
 the hyperparameter optimization,
 learning curves, n-fold cross-validations,
 saving parameters and results
 

 Args:
     version: user defined name
     scoring: evaluation metric 
     cv: # of cross-validation
     n_jobs: for running in parallel
     prop: property to train
     do_cv: whether to perform cross validation
            
 """
    name = str(version) + str("_") + str(prop)
    # Make a directory for storing results
    dir_name = str(os.getcwd()) + str("/") + str(name)
    if not os.path.exists(dir_name):
        os.makedirs(dir_name)
    os.chdir(dir_name)

    info = {}
    tmp_time = time.time()

    # STEP-1: Data for a particular model

    x, y, jid = jdata(prop=prop)

    # Toy example with boston data
    # boston=load_boston()
    # x, y = boston['data'], boston['target']

    # STEP-2: Splitting the data
    # 90-10% split for train test
    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        x, y, jid, random_state=1, test_size=0.1
    )

    # further split for boosting iteration convergence
    X_train1, X_test1, y_train1, y_test1 = train_test_split(
        X_train, y_train, random_state=1, test_size=0.1
    )
    print("lenx len y", len(x[0]), len(y))

    # STEP-3: GBM model with converged iterations

    model = get_lgbm(X_train1, X_test1, y_train1, y_test1, cv, n_jobs, scoring)

    model.fit(X_train, y_train)
    print(model, model.named_steps["est"].n_estimators)
    print("model", model.__dict__)
    # info['model']=model.__dict__

    # STEP-4: Predict on 10% held data and  accuracy

    pred = model.predict(X_test)
    reg_sc = regr_scores(y_test, pred)

    print("reg_sc", reg_sc)
    info["reg_sc"] = reg_sc
    info["y"] = y
    info["time"] = time.time() - tmp_time
    info["jid_test"] = jid_test

    model.fit(X_train1, y_train1)
    print("Plot feature importances...")

    # STEP-5: Feature importance
    feat_imp = model.named_steps["est"].feature_importances_  # feature_importances_
    feat_imp = np.array([float(i) / float(np.sum(feat_imp)) for i in feat_imp])
    print("feat_imp", name, feat_imp, len(feat_imp))

    # Since variance threshold removed some features
    keep_indices2 = model.named_steps["vart"].get_support(indices=True)
    info["keep_indices_sel"] = keep_indices2

    print("keep_indices2", keep_indices2, len(keep_indices2))

    indices = np.argsort(feat_imp)[::-1]
    xs = feat_imp[indices]
    ys = keep_indices2[indices]

    print("xs,ys", len(xs), len(ys))
    data_imp = []
    f = open("feat_names.json", "r")
    names = json.load(f)
    f.close()

    for f in range(len(xs)):
        print(f, xs[f], names[ys[f]])
        data_imp.append([f, xs[f], names[ys[f]]])
    info["feature_importance"] = data_imp
    # STEP-6: N-fold cross-validation prediction

    scores = cross_val_score(
        model, x, y, cv=cv, n_jobs=n_jobs, scoring="mean_absolute_error"
    )
    print("MAE-Score: %0.2f (+/- %0.2f)" % (scores.mean(), scores.std() * 2))
    info["cross_val_scores"] = scores

    fname = str(name) + str("-GB-CV.png")
    predicted = cross_val_predict(model, x, y, cv=cv, n_jobs=n_jobs)
    fig, ax = plt.subplots()
    ax.set_xlabel("DFT prediction")
    ax.set_ylabel("ML prediction")
    print("cv_predicted")
    cvpred = {}
    cvpred["cross_val_y"] = y
    cvpred["cross_val_predicted"] = predicted
    info["cvpredict_result"] = cvpred

    ax.scatter(y, predicted, edgecolors=(0, 0, 0))
    plt.savefig(fname)
    plt.close()

    # STEP-7: Plot learning curve

    mem_learn = plot_learning_curve(
        estimator=model, scoring=scoring, fname=name, title="", X=x, y=y, n_jobs=n_jobs
    )
    info["learning_curve_data"] = mem_learn

    # STEP-8: Store ML parameters with pickle and joblib

    model.fit(x, y)
    filename = str("pickle2-") + str(name) + str(".pk")
    pickle.dump(model, open(filename, "wb"))
    filename = str("joblib2-") + str(name) + str(".pkl")
    joblib.dump(model, filename)

    # STEP-9: Store data in a json file

    file = str(name) + "_info.json"
    f = open(file, "w")
    f.write(json.dumps(info, cls=MontyEncoder, indent=4))
    f.close()
    os.chdir("../")


if __name__ == "__main__":
    # This may take long time
    # run(version='version_1',scoring='neg_mean_absolute_error',cv=5,n_jobs=1,prop='op_gap',do_cv=False)

    # smaller test fit model
    model = lgb(
        n_estimators=100,
        learning_rate=0.1,
        max_depth=5,
        num_leaves=100,
        objective="regression",
        n_jobs=-1,
        verbose=-1,
    )
    x, y, jid = jdata(prop="form_enp")
    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        x, y, jid, random_state=1, test_size=0.1
    )
    len(X_train), len(X_test)

    # Let's take 500 of training set as a quick example
    X = X_train[0:500]
    Y = y_train[0:500]
    model.fit(X, Y)

    info = {}
    tmp_time = time.time()

    # STEP-1: Data for a particular model

    x, y, jid = jdata(prop=prop)

    # Toy example with boston data
    # boston=load_boston()
    # x, y = boston['data'], boston['target']

    # STEP-2: Splitting the data
    # 90-10% split for train test
    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        x, y, jid, random_state=1, test_size=0.1
    )

    # further split for boosting iteration convergence
    X_train1, X_test1, y_train1, y_test1 = train_test_split(
        X_train, y_train, random_state=1, test_size=0.1
    )
    print("lenx len y", len(x[0]), len(y))

    # STEP-3: GBM model with converged iterations

    model = get_lgbm(X_train1, X_test1, y_train1, y_test1, cv, n_jobs, scoring)

    model.fit(X_train, y_train)
    print(model, model.named_steps["est"].n_estimators)
    print("model", model.__dict__)
    # info['model']=model.__dict__

    # STEP-4: Predict on 10% held data and  accuracy

    pred = model.predict(X_test)
    reg_sc = regr_scores(y_test, pred)

    print("reg_sc", reg_sc)
    info["reg_sc"] = reg_sc
    info["y"] = y
    info["time"] = time.time() - tmp_time
    info["jid_test"] = jid_test

    model.fit(X_train1, y_train1)
    print("Plot feature importances...")

    # STEP-5: Feature importance
    feat_imp = model.named_steps["est"].feature_importances_  # feature_importances_
    feat_imp = np.array([float(i) / float(np.sum(feat_imp)) for i in feat_imp])
    print("feat_imp", name, feat_imp, len(feat_imp))

    # Since variance threshold removed some features
    keep_indices2 = model.named_steps["vart"].get_support(indices=True)
    info["keep_indices_sel"] = keep_indices2

    print("keep_indices2", keep_indices2, len(keep_indices2))

    indices = np.argsort(feat_imp)[::-1]
    xs = feat_imp[indices]
    ys = keep_indices2[indices]

    print("xs,ys", len(xs), len(ys))
    data_imp = []
    f = open("feat_names.json", "r")
    names = json.load(f)
    f.close()

    for f in range(len(xs)):
        print(f, xs[f], names[ys[f]])
        data_imp.append([f, xs[f], names[ys[f]]])
    info["feature_importance"] = data_imp
    # STEP-6: N-fold cross-validation prediction

    scores = cross_val_score(
        model, x, y, cv=cv, n_jobs=n_jobs, scoring="mean_absolute_error"
    )
    print("MAE-Score: %0.2f (+/- %0.2f)" % (scores.mean(), scores.std() * 2))
    info["cross_val_scores"] = scores

    fname = str(name) + str("-GB-CV.png")
    predicted = cross_val_predict(model, x, y, cv=cv, n_jobs=n_jobs)
    fig, ax = plt.subplots()
    ax.set_xlabel("DFT prediction")
    ax.set_ylabel("ML prediction")
    print("cv_predicted")
    cvpred = {}
    cvpred["cross_val_y"] = y
    cvpred["cross_val_predicted"] = predicted
    info["cvpredict_result"] = cvpred

    ax.scatter(y, predicted, edgecolors=(0, 0, 0))
    plt.savefig(fname)
    plt.close()

    # STEP-7: Plot learning curve

    mem_learn = plot_learning_curve(
        estimator=model, scoring=scoring, fname=name, title="", X=x, y=y, n_jobs=n_jobs
    )
    info["learning_curve_data"] = mem_learn

    # STEP-8: Store ML parameters with pickle and joblib

    model.fit(x, y)
    filename = str("pickle2-") + str(name) + str(".pk")
    pickle.dump(model, open(filename, "wb"))
    filename = str("joblib2-") + str(name) + str(".pkl")
    joblib.dump(model, filename)

    # STEP-9: Store data in a json file

    file = str(name) + "_info.json"
    f = open(file, "w")
    f.write(json.dumps(info, cls=MontyEncoder, indent=4))
    f.close()
    os.chdir("../")


if __name__ == "__main__":
    # This may take long time
    # run(version='version_1',scoring='neg_mean_absolute_error',cv=5,n_jobs=1,prop='op_gap',do_cv=False)

    # smaller test fit model
    model = lgb(
        n_estimators=100,
        learning_rate=0.1,
        max_depth=5,
        num_leaves=100,
        objective="regression",
        n_jobs=-1,
        verbose=-1,
    )
    x, y, jid = jdata(prop="form_enp")
    X_train, X_test, y_train, y_test, jid_train, jid_test = train_test_split(
        x, y, jid, random_state=1, test_size=0.1
    )
    len(X_train), len(X_test)

    # Let's take 500 of training set as a quick example
    X = X_train[0:500]
    Y = y_train[0:500]
    model.fit(X, Y)

    pred = model.predict(X_test)
    reg_sc = regr_scores(y_test, pred)



from pymatgen.core.structure import Structure
import numpy as np
from pymatgen.analysis.elasticity.elastic import ElasticTensor
import os


def elastic_props(outcar="", vacuum=False):
    """
    Obtain elastic tensor and calculate related properties

    Args:
        outcar: OUTCAR file path
        vacuum: whether the structure has vaccum such as 2D materials
        for vacuum structures bulk and shear mod. needs extra attenstion
        and elastic tensor are in Nm^-1 rather than GPa
    Returns:
          info: data for elastic tensor (in string and object representation), bulk, shear modulus, and phonon modes
    """
    if vacuum == True:
        contcar = Structure.from_file(str(outcar).replace("OUTCAR", "POSCAR"))
        ratio_c = 0.1 * float(
            abs(contcar.lattice.matrix[2][2])
        )  # *(10**9)*(10**-10) #N/m unit
    el_tens = "na"
    KV = "na"
    GV = "na"
    spin = "na"
    info = {}
    ratio_c = 1.0
    v = open(outcar, "r")
    lines = v.read().splitlines()
    for i, line in enumerate(lines):
        if "TOTAL ELASTIC MODULI (kBar)" in line:
            c11 = lines[i + 3].split()[1]
            c12 = lines[i + 3].split()[2]
            c13 = lines[i + 3].split()[3]
            c14 = lines[i + 3].split()[4]
            c15 = lines[i + 3].split()[5]
            c16 = lines[i + 3].split()[6]
            c21 = lines[i + 4].split()[1]
            c22 = lines[i + 4].split()[2]
            c23 = lines[i + 4].split()[3]
            c24 = lines[i + 4].split()[4]
            c25 = lines[i + 4].split()[5]
            c26 = lines[i + 4].split()[6]
            c31 = lines[i + 5].split()[1]
            c32 = lines[i + 5].split()[2]
            c33 = lines[i + 5].split()[3]
            c34 = lines[i + 5].split()[4]
            c35 = lines[i + 5].split()[5]
            c36 = lines[i + 5].split()[6]
            c41 = lines[i + 6].split()[1]
            c42 = lines[i + 6].split()[2]
            c43 = lines[i + 6].split()[3]
            c44 = lines[i + 6].split()[4]
            c45 = lines[i + 6].split()[5]
            c46 = lines[i + 6].split()[6]
            c51 = lines[i + 7].split()[1]
            c52 = lines[i + 7].split()[2]
            c53 = lines[i + 7].split()[3]
            c54 = lines[i + 7].split()[4]
            c55 = lines[i + 7].split()[5]
            c56 = lines[i + 7].split()[6]
            c61 = lines[i + 8].split()[1]
            c62 = lines[i + 8].split()[2]
            c63 = lines[i + 8].split()[3]
            c64 = lines[i + 8].split()[4]
            c65 = lines[i + 8].split()[5]
            c66 = lines[i + 8].split()[6]
            c11 = round(ratio_c * float(c11) / float(10), 1)
            c12 = round(ratio_c * float(c12) / float(10), 1)
            c13 = round(ratio_c * float(c13) / float(10), 1)
            c14 = round(ratio_c * float(c14) / float(10), 1)
            c15 = round(ratio_c * float(c15) / float(10), 1)
            c16 = round(ratio_c * float(c16) / float(10), 1)
            c21 = round(ratio_c * float(c21) / float(10), 1)
            c22 = round(ratio_c * float(c22) / float(10), 1)
            c23 = round(ratio_c * float(c23) / float(10), 1)
            c24 = round(ratio_c * float(c24) / float(10), 1)
            c25 = round(ratio_c * float(c25) / float(10), 1)
            c26 = round(ratio_c * float(c26) / float(10), 1)
            c31 = round(float(c31) / float(10), 1)
            c32 = round(float(c32) / float(10), 1)
            c33 = round(float(c33) / float(10), 1)
            c34 = round(float(c34) / float(10), 1)
            c35 = round(float(c35) / float(10), 1)
            c36 = round(float(c36) / float(10), 1)
            c41 = round(float(c41) / float(10), 1)
            c42 = round(float(c42) / float(10), 1)
            c43 = round(float(c43) / float(10), 1)
            c44 = round(float(c44) / float(10), 1)
            c45 = round(float(c45) / float(10), 1)
            c46 = round(float(c46) / float(10), 1)
            c51 = round(float(c51) / float(10), 1)
            c52 = round(float(c52) / float(10), 1)
            c53 = round(float(c53) / float(10), 1)
            c54 = round(float(c54) / float(10), 1)
            c55 = round(float(c55) / float(10), 1)
            c56 = round(float(c56) / float(10), 1)
            c61 = round(float(c61) / float(10), 1)
            c62 = round(float(c62) / float(10), 1)
            c63 = round(float(c63) / float(10), 1)
            c64 = round(float(c64) / float(10), 1)
            c65 = round(float(c65) / float(10), 1)
            c66 = round(float(c66) / float(10), 1)
            KV = float((c11 + c22 + c33) + 2 * (c12 + c23 + c31)) / float(9)
            GV = float(
                (c11 + c22 + c33) - (c12 + c23 + c31) + 3 * (c44 + c55 + c66)
            ) / float(15)
            KV = round(KV, 3)
            GV = round(GV, 3)
            break
    v.close()

    # Convenient string representation for storage

    el_tens = (
        str(c11)
        + str(",")
        + str(c12)
        + str(",")
        + str(c13)
        + str(",")
        + str(c14)
        + str(",")
        + str(c15)
        + str(",")
        + str(c16)
        + str(",")
        + str(c21)
        + str(",")
        + str(c22)
        + str(",")
        + str(c23)
        + str(",")
        + str(c24)
        + str(",")
        + str(c25)
        + str(",")
        + str(c26)
        + str(",")
        + str(c31)
        + str(",")
        + str(c32)
        + str(",")
        + str(c33)
        + str(",")
        + str(c34)
        + str(",")
        + str(c35)
        + str(",")
        + str(c36)
        + str(",")
        + str(c41)
        + str(",")
        + str(c42)
        + str(",")
        + str(c43)
        + str(",")
        + str(c44)
        + str(",")
        + str(c45)
        + str(",")
        + str(c46)
        + str(",")
        + str(c51)
        + str(",")
        + str(c52)
        + str(",")
        + str(c53)
        + str(",")
        + str(c54)
        + str(",")
        + str(c55)
        + str(",")
        + str(c56)
        + str(",")
        + str(c61)
        + str(",")
        + str(c62)
        + str(",")
        + str(c63)
        + str(",")
        + str(c64)
        + str(",")
        + str(c65)
        + str(",")
        + str(c66)
    )

    cij = np.empty((6, 6), dtype=float)
    elast = np.array(el_tens.split(","), dtype="float")
    count = 0
    for ii in range(6):
        for jj in range(6):
            cij[ii][jj] = elast[count]
    el_tens2 = ElasticTensor.from_voigt(cij)

    modes = []
    try:
        for i in lines:
            if "cm-1" in i and "meV" in i:

                mod = float(i.split()[7])
                if "f/i" in i:
                    mod = mod * -1
                if mod not in modes:
                    modes.append(float(mod))
    except:
        pass

    info["el_tens_str"] = el_tens
    info["el_tens_obj"] = el_tens2
    info["KV"] = KV
    info["GV"] = GV
    info["modes"] = modes

    return info


def get_et(elast_str=""):
    cij = np.empty((6, 6), dtype=float)
    elast = np.array(elast_str.split(","), dtype="float")
    count = 0
    for ii in range(6):
        for jj in range(6):
            cij[ii][jj] = elast[count]
            count = count + 1
    et = ElasticTensor.from_voigt(cij)
    return et


if __name__ == "__main__":
    out = path = str(
        os.path.join(
            os.path.dirname(__file__),
            "../vasp/examples/SiOptb88/MAIN-ELASTIC-bulk@mp_149/OUTCAR",
        )
    )
    ep = elastic_props(out)
    print(ep)



from __future__ import unicode_literals, print_function

# Author: Marnik Bercx (University of Antwerp), Kamal Choudhary (NIST)
# Forked and adjusted from https://github.com/ldwillia/SL3ME

"""
Calculate spectroscopy limited maximum efficiency (SLME) given dielectric function data
"""

import os
import numpy as np
import matplotlib as plt
import scipy.constants as constants
from scipy.integrate import simps
import matplotlib.pyplot as plt

plt.switch_backend("agg")
import glob, os, math
from pymatgen.io.vasp.outputs import Vasprun
from numpy import loadtxt, arange, logspace
from math import pi, sqrt
from scipy.constants import physical_constants, speed_of_light
from pymatgen.electronic_structure.core import Spin


eV_to_recip_cm = 1.0 / (
    physical_constants["Planck constant in eV s"][0] * speed_of_light * 1e2
)


def nelec_out(out=""):
    f = open(out, "r")
    lines = f.read().splitlines()
    f.close()
    for i in lines:
        if "NELECT =" in i:
            nelec = int(
                float(i.split()[2])
            )  # int(i.split('NELECT =')[1].split('total number of electrons')[0])
    return nelec


def get_dir_indir_gap(run=""):
    """
 Get direct and indirect bandgaps for a vasprun.xml
 """

    v = Vasprun(run)
    bandstructure = v.get_band_structure()
    dir_gap = bandstructure.get_direct_band_gap()
    indir_gap = bandstructure.get_band_gap()["energy"]
    return dir_gap, indir_gap


def matrix_eigvals(matrix):
    """
    Calculate the eigenvalues of a matrix.
    Args:
        matrix (np.array): The matrix to diagonalise.
    Returns:
        (np.array): Array of the matrix eigenvalues.
    """
    eigvals, eigvecs = np.linalg.eig(matrix)
    return eigvals


def to_matrix(xx, yy, zz, xy, yz, xz):
    """
    Convert a list of matrix components to a symmetric 3x3 matrix.
    Inputs should be in the order xx, yy, zz, xy, yz, xz.
    Args:
        xx (float): xx component of the matrix.
        yy (float): yy component of the matrix.
        zz (float): zz component of the matrix.
        xy (float): xy component of the matrix.
        yz (float): yz component of the matrix.
        xz (float): xz component of the matrix.
    Returns:
        (np.array): The matrix, as a 3x3 numpy array.
    """
    matrix = np.array([[xx, xy, xz], [xy, yy, yz], [xz, yz, zz]])
    return matrix


def parse_dielectric_data(data):
    """
    Convert a set of 2D vasprun formatted dielectric data to
    the eigenvalues of each corresponding 3x3 symmetric numpy matrices.
    Args:
        data (list): length N list of dielectric data. Each entry should be
                     a list of ``[xx, yy, zz, xy, xz, yz ]`` dielectric
                     tensor elements.
    Returns:
        (np.array):  a Nx3 numpy array. Each row contains the eigenvalues
                     for the corresponding row in `data`.
    """
    return np.array([matrix_eigvals(to_matrix(*e)) for e in data])


def absorption_coefficient(dielectric):
    """
    Calculate the optical absorption coefficient from an input set of
    pymatgen vasprun dielectric constant data.
    Args:
        dielectric (list): A list containing the dielectric response function
                           in the pymatgen vasprun format.
                           | element 0: list of energies
                           | element 1: real dielectric tensors, in ``[xx, yy, zz, xy, xz, yz]`` format.
                           | element 2: imaginary dielectric tensors, in ``[xx, yy, zz, xy, xz, yz]`` format.

    Returns:
        (np.array): absorption coefficient using eV as frequency units (cm^-1).
    Notes:
        The absorption coefficient is calculated as
        .. math:: \\alpha = \\frac{2\sqrt{2} \pi}{\lambda} \sqrt{-\epsilon_1+\sqrt{\epsilon_1^2+\epsilon_2^2}}
    """
    energies_in_eV = np.array(dielectric[0])
    real_dielectric = parse_dielectric_data(dielectric[1])
    imag_dielectric = parse_dielectric_data(dielectric[2])
    epsilon_1 = np.mean(real_dielectric, axis=1)
    epsilon_2 = np.mean(imag_dielectric, axis=1)
    return (
        energies_in_eV,
        (
            2.0
            * np.sqrt(2.0)
            * pi
            * eV_to_recip_cm
            * energies_in_eV
            * np.sqrt(-epsilon_1 + np.sqrt(epsilon_1 ** 2 + epsilon_2 ** 2))
        ),
    )


def optics(ru=""):
    dirgap, indirgap = get_dir_indir_gap(ru)

    run = Vasprun(ru, occu_tol=1e-2)
    new_en, new_abs = absorption_coefficient(run.dielectric)
    return (
        np.array(new_en, dtype=np.float64),
        np.array(new_abs, dtype=np.float64),
        dirgap,
        indirgap,
    )


def calculate_SQ(bandgap_ev, temperature=300, fr=1, plot_current_voltage=False):
    """
    Args:
        bandgap_ev: bandga in electron-volt
        temperature: temperature in K
    Returns:
         
    """

    # Defining constants for tidy equations
    c = constants.c  # speed of light, m/s
    h = constants.h  # Planck's constant J*s (W)
    h_e = constants.h / constants.e  # Planck's constant eV*s
    k = constants.k  # Boltzmann's constant J/K
    k_e = constants.k / constants.e  # Boltzmann's constant eV/K
    e = constants.e  # Coulomb

    # Load the Air Mass 1.5 Global tilt solar spectrum
    solar_spectrum_data_file = str(
        os.path.join(os.path.dirname(__file__), "am1.5G.dat")
    )
    solar_spectra_wavelength, solar_spectra_irradiance = np.loadtxt(
        solar_spectrum_data_file, usecols=[0, 1], unpack=True, skiprows=2
    )

    solar_spectra_wavelength_meters = solar_spectra_wavelength * 1e-9

    # need to convert solar irradiance from Power/m**2(nm) into
    # photon#/s*m**2(nm) power is Watt, which is Joule / s
    # E = hc/wavelength
    # at each wavelength, Power * (wavelength(m)/(h(Js)*c(m/s))) = ph#/s
    solar_spectra_photon_flux = solar_spectra_irradiance * (
        solar_spectra_wavelength_meters / (h * c)
    )

    ### Calculation of total solar power incoming
    power_in = simps(solar_spectra_irradiance, solar_spectra_wavelength)

    # calculation of blackbody irradiance spectra
    # units of W/(m**3), different than solar_spectra_irradiance!!! (This
    # is intentional, it is for convenience)
    blackbody_irradiance = (
        2.0 * h * c ** 2 / (solar_spectra_wavelength_meters ** 5)
    ) * (
        1.0
        / ((np.exp(h * c / (solar_spectra_wavelength_meters * k * temperature))) - 1.0)
    )

    # I've removed a pi in the equation above - Marnik Bercx

    # now to convert the irradiance from Power/m**2(m) into photon#/s*m**2(m)
    blackbody_photon_flux = blackbody_irradiance * (
        solar_spectra_wavelength_meters / (h * c)
    )

    # absorbance interpolation onto each solar spectrum wavelength
    from numpy import interp

    # Get the bandgap in wavelength in meters
    bandgap_wavelength = h_e * c / bandgap_ev

    # Only take the part of the wavelength-dependent solar spectrum and
    # blackbody spectrum below the bandgap wavelength
    bandgap_index = np.searchsorted(solar_spectra_wavelength_meters, bandgap_wavelength)

    bandgap_irradiance = interp(
        np.array([bandgap_wavelength]),
        solar_spectra_wavelength_meters,
        solar_spectra_photon_flux,
    )

    bandgap_blackbody = (
        (2.0 * h * c ** 2 / (bandgap_wavelength ** 5))
        * (1.0 / ((np.exp(h * c / (bandgap_wavelength * k * temperature))) - 1.0))
        * (bandgap_wavelength / (h * c))
    )

    integration_wavelength = np.concatenate(
        (
            solar_spectra_wavelength_meters[:bandgap_index],
            np.array([bandgap_wavelength]),
        ),
        axis=0,
    )

    integration_solar_flux = np.concatenate(
        (solar_spectra_photon_flux[:bandgap_index], bandgap_irradiance), axis=0
    )

    integration_blackbody = np.concatenate(
        (blackbody_photon_flux[:bandgap_index], np.array([bandgap_blackbody])), axis=0
    )

    #  Numerically integrating irradiance over wavelength array
    # Note: elementary charge, not math e!  ## units of A/m**2   W/(V*m**2)
    J_0_r = e * np.pi * simps(integration_blackbody, integration_wavelength)

    J_0 = J_0_r / fr

    #  Numerically integrating irradiance over wavelength array
    # elementary charge, not math e!  ### units of A/m**2   W/(V*m**2)
    J_sc = e * simps(integration_solar_flux * 1e9, integration_wavelength)

    #    J[i] = J_sc - J_0*(1 - exp( e*V[i]/(k*T) ) )
    #   #This formula from the original paper has a typo!!
    #    J[i] = J_sc - J_0*(exp( e*V[i]/(k*T) ) - 1)
    #   #Bercx chapter and papers have the correct formula (well,
    #   the correction on one paper)
    def J(V):
        J = J_sc - J_0 * (np.exp(e * V / (k * temperature)) - 1.0)
        return J

    def power(V):
        p = J(V) * V
        return p

    # A more primitive, but perfectly robust way of getting a reasonable
    # estimate for the maximum power.
    test_voltage = 0
    voltage_step = 0.001
    while power(test_voltage + voltage_step) > power(test_voltage):
        test_voltage += voltage_step

    max_power = power(test_voltage)

    # Calculate the maximized efficience
    efficiency = max_power / power_in

    # This segment isn't needed for functionality at all, but can display a
    # plot showing how the maximization of power by choosing the optimal
    # voltage value works
    if plot_current_voltage:
        V = np.linspace(0, 2, 200)
        plt.plot(V, J(V))
        plt.plot(V, power(V), linestyle="--")
        plt.show()
        print(max_power)

    return efficiency


def slme(
    material_energy_for_absorbance_data,
    material_absorbance_data,
    material_direct_allowed_gap,
    material_indirect_gap,
    thickness=50e-6,
    temperature=293.15,
    absorbance_in_inverse_centimeters=False,
    cut_off_absorbance_below_direct_allowed_gap=True,
    plot_current_voltage=False,
):
    """
    Calculate the

    IMPORTANT NOTES:
    1) Material calculated absorbance is assumed to be in m^-1, not cm^-1!
        (Most sources will provide absorbance in cm^-1, so be careful.)

    2) The default is to remove absorbance below the direct allowed gap.
        This is for dealing with broadening applied in DFT absorbance
        calculations. Probably not desired for experimental data.

    3) We can calculate at different temperatures if we want to, but 25 C /
        293.15 K is the standard temperature assumed if not specified

    4) If absorbance is in cm^-1, multiply values by 100 to match units
        assumed in code

    Args:
        material_energy_for_absorbance_data:
        material_absorbance_data:
        material_direct_allowed_gap:
        material_indirect_gap:
        thickness:
        temperature:
        absorbance_in_inverse_centimeters:
        cut_off_absorbance_below_direct_allowed_gap:
        plot_current_voltage:

    Returns:
        The calculated maximum efficiency.

    """

    # Defining constants for tidy equations
    c = constants.c  # speed of light, m/s
    h = constants.h  # Planck's constant J*s (W)
    h_e = constants.h / constants.e  # Planck's constant eV*s
    k = constants.k  # Boltzmann's constant J/K
    k_e = constants.k / constants.e  # Boltzmann's constant eV/K
    e = constants.e  # Coulomb

    # Make sure the absorption coefficient has the right units (m^{-1})
    if absorbance_in_inverse_centimeters:
        material_absorbance_data = material_absorbance_data * 100

    # Load the Air Mass 1.5 Global tilt solar spectrum
    solar_spectrum_data_file = str(
        os.path.join(os.path.dirname(__file__), "am1.5G.dat")
    )

    solar_spectra_wavelength, solar_spectra_irradiance = np.loadtxt(
        solar_spectrum_data_file, usecols=[0, 1], unpack=True, skiprows=2
    )

    solar_spectra_wavelength_meters = solar_spectra_wavelength * 1e-9

    delta = material_direct_allowed_gap - material_indirect_gap
    fr = np.exp(-delta / (k_e * temperature))

    # need to convert solar irradiance from Power/m**2(nm) into
    # photon#/s*m**2(nm) power is Watt, which is Joule / s
    # E = hc/wavelength
    # at each wavelength, Power * (wavelength(m)/(h(Js)*c(m/s))) = ph#/s
    solar_spectra_photon_flux = solar_spectra_irradiance * (
        solar_spectra_wavelength_meters / (h * c)
    )

    # Calculation of total solar power incoming
    power_in = simps(solar_spectra_irradiance, solar_spectra_wavelength)

    # calculation of blackbody irradiance spectra
    # units of W/(m**3), different than solar_spectra_irradiance!!! (This
    # is intentional, it is for convenience)
    blackbody_irradiance = (
        2.0 * h * c ** 2 / (solar_spectra_wavelength_meters ** 5)
    ) * (
        1.0
        / ((np.exp(h * c / (solar_spectra_wavelength_meters * k * temperature))) - 1.0)
    )

    # I've removed a pi in the equation above - Marnik Bercx

    # now to convert the irradiance from Power/m**2(m) into photon#/s*m**2(m)
    blackbody_photon_flux = blackbody_irradiance * (
        solar_spectra_wavelength_meters / (h * c)
    )

    # units of nm
    material_wavelength_for_absorbance_data = (
        (c * h_e) / (material_energy_for_absorbance_data + 0.00000001)
    ) * 10 ** 9

    # absorbance interpolation onto each solar spectrum wavelength
    from scipy.interpolate import interp1d

    # creates cubic spline interpolating function, set up to use end values
    #  as the guesses if leaving the region where data exists
    material_absorbance_data_function = interp1d(
        material_wavelength_for_absorbance_data,
        material_absorbance_data,
        kind="cubic",
        fill_value=(material_absorbance_data[0], material_absorbance_data[-1]),
        bounds_error=False,
    )

    material_interpolated_absorbance = np.zeros(len(solar_spectra_wavelength_meters))
    for i in range(0, len(solar_spectra_wavelength_meters)):
        ## Cutting off absorption data below the gap. This is done to deal
        # with VASPs broadening of the calculated absorption data

        if (
            solar_spectra_wavelength[i]
            < 1e9 * ((c * h_e) / material_direct_allowed_gap)
            or cut_off_absorbance_below_direct_allowed_gap == False
        ):
            material_interpolated_absorbance[i] = material_absorbance_data_function(
                solar_spectra_wavelength[i]
            )

    absorbed_by_wavelength = 1.0 - np.exp(
        -2.0 * material_interpolated_absorbance * thickness
    )

    #  Numerically integrating irradiance over wavelength array
    # Note: elementary charge, not math e!  ## units of A/m**2   W/(V*m**2)
    J_0_r = (
        e
        * np.pi
        * simps(
            blackbody_photon_flux * absorbed_by_wavelength,
            solar_spectra_wavelength_meters,
        )
    )

    J_0 = J_0_r / fr

    #  Numerically integrating irradiance over wavelength array
    # elementary charge, not math e!  ### units of A/m**2   W/(V*m**2)
    J_sc = e * simps(
        solar_spectra_photon_flux * absorbed_by_wavelength, solar_spectra_wavelength
    )

    #    J[i] = J_sc - J_0*(1 - exp( e*V[i]/(k*T) ) )
    #   #This formula from the original paper has a typo!!
    #    J[i] = J_sc - J_0*(exp( e*V[i]/(k*T) ) - 1)
    #   #Bercx chapter and papers have the correct formula (well,
    #   the correction on one paper)
    def J(V):
        J = J_sc - J_0 * (np.exp(e * V / (k * temperature)) - 1.0)
        return J

    def power(V):
        p = J(V) * V
        return p

    # A more primitive, but perfectly robust way of getting a reasonable
    # estimate for the maximum power.
    test_voltage = 0
    voltage_step = 0.001
    while power(test_voltage + voltage_step) > power(test_voltage):
        test_voltage += voltage_step

    max_power = power(test_voltage)

    # Calculate the maximized efficience
    efficiency = max_power / power_in

    # This segment isn't needed for functionality at all, but can display a
    # plot showing how the maximization of power by choosing the optimal
    # voltage value works
    if plot_current_voltage:
        V = np.linspace(0, 2, 200)
        plt.plot(V, J(V))
        plt.plot(V, power(V), linestyle="--")
        plt.savefig("pp.png")
        plt.close()
        # print(power(V_Pmax))

    return efficiency


if __name__ == "__main__":
    path = str(
        os.path.join(
            os.path.dirname(__file__),
            "..",
            "vasp",
            "examples",
            "SiOptb88",
            "MAIN-MBJ-bulk@mp_149",
            "vasprun.xml",
        )
    )
    en, abz, dirgap, indirgap = optics(path)
    abz = abz * 100.0
    eff = slme(en, abz, indirgap, indirgap, plot_current_voltage=False)
    print("SLME", 100 * eff)
    eff = calculate_SQ(indirgap)
    print("SQ", 100 * eff)
